import errno
import os


def makedirs(path, mode=0o777, exist_ok=False):
    try:
        os.makedirs(path)
    except OSError as e:
        if not exist_ok or e.errno != errno.EEXIST or not os.path.isdir(path):
            raise


class pushd(object):
    def __init__(self, dirname):
        self.cwd = dirname

    def __enter__(self):
        self.old = os.getcwd()
        os.chdir(self.cwd)
        return self

    def __exit__(self, type, value, traceback):
        os.chdir(self.old)



import os
import re
import subprocess
from packaging.version import Version

from ... import path
from .syntax import *
from ...platforms import which


def version(env=os.environ):
    try:
        make = which(env.get('MAKE', ['make', 'gmake']), env)
        output = subprocess.check_output(
            [make, '--version'],
            universal_newlines=True
        )
        m = re.match(r'GNU Make ([\d\.]+)', output)
        if m:
            return Version(m.group(1))
    except IOError:
        pass
    return None

priority = 2

_rule_handlers = {}
_pre_rules = []
_post_rules = []

dir_sentinel = '.dir'


def rule_handler(rule_name):
    def decorator(fn):
        _rule_handlers[rule_name] = fn
        return fn
    return decorator


def pre_rule(fn):
    _pre_rules.append(fn)
    return fn


def post_rule(fn):
    _post_rules.append(fn)
    return fn


def write(env, build_inputs):
    buildfile = Makefile()
    buildfile.variable(path_vars[path.Root.srcdir], env.srcdir, Section.path)
    for i in path.InstallRoot:
        buildfile.variable(path_vars[i], env.install_dirs[i], Section.path)

    for i in _pre_rules:
        i(build_inputs, buildfile, env)
    for e in build_inputs.edges:
        _rule_handlers[type(e)](e, build_inputs, buildfile, env)
    for i in _post_rules:
        i(build_inputs, buildfile, env)

    with open(env.builddir.append('Makefile').string(), 'w') as out:
        buildfile.write(out)


def cmd_var(cmd, buildfile):
    name = cmd.command_var.upper()
    return buildfile.variable(name, cmd.command, Section.command, True)


def flags_vars(name, value, buildfile):
    name = name.upper()
    gflags = buildfile.variable('GLOBAL_' + name, value, Section.flags, True)
    flags = buildfile.target_variable(name, gflags, True)
    return gflags, flags


@post_rule
def directory_rule(build_inputs, buildfile, env):
    mkdir_p = env.tool('mkdir_p')
    pattern = Pattern(os.path.join('%', dir_sentinel))
    path = Function('patsubst', pattern, Pattern('%'), var('@'), quoted=True)

    buildfile.rule(
        target=pattern,
        recipe=[
            silent(mkdir_p(cmd_var(mkdir_p, buildfile), path)),
            silent(['touch', qvar('@')])
        ]
    )



import os
import platform
import subprocess

from .iterutils import iterate
from .path import Path, Root, InstallRoot


def platform_name():
    name = platform.system().lower()
    if name == 'windows':
        try:
            uname = subprocess.check_output(
                'uname', universal_newlines=True
            ).lower()
            if uname.startswith('cygwin'):
                name = 'cygwin'
        except WindowsError:
            pass
    return name


# Platform objects are primarily intended to represent information about the
# target platform for a build. Currently, we store some source platform
# information here as well (e.g. include/library dirs). In the future, when we
# support cross-compilation, stuff like that should be moved elsewhere.
class Platform(object):
    def __init__(self, name):
        self.name = name


class PosixPlatform(Platform):
    @property
    def flavor(self):
        return 'posix'

    @property
    def executable_ext(self):
        return ''

    @property
    def shared_library_ext(self):
        return '.so'

    @property
    def has_import_library(self):
        return False

    @property
    def has_rpath(self):
        return False

    @property
    def include_dirs(self):
        return ['/usr/local/include', '/usr/include']

    @property
    def lib_dirs(self):
        return ['/usr/local/lib', '/lib', '/usr/lib']

    @property
    def install_dirs(self):
        return {
            InstallRoot.prefix:     Path('/usr/local', Root.absolute),
            InstallRoot.bindir:     Path('bin', InstallRoot.prefix),
            InstallRoot.libdir:     Path('lib', InstallRoot.prefix),
            InstallRoot.includedir: Path('include', InstallRoot.prefix),
        }


class LinuxPlatform(PosixPlatform):
    @property
    def has_rpath(self):
        return True


class DarwinPlatform(PosixPlatform):
    @property
    def shared_library_ext(self):
        return '.dylib'


class WindowsPlatform(Platform):
    @property
    def flavor(self):
        return 'windows'

    @property
    def executable_ext(self):
        return '.exe'

    @property
    def shared_library_ext(self):
        return '.dll'

    @property
    def has_import_library(self):
        return True

    @property
    def has_rpath(self):
        return False

    @property
    def include_dirs(self):
        # Windows doesn't have standard include dirs; for MSVC, this will get
        # pulled in from the $INCLUDE environment variable.
        return []

    @property
    def lib_dirs(self):
        # Windows doesn't have standard library dirs; for MSVC, this will get
        # pulled in from the $LIB environment variable.
        return []

    @property
    def install_dirs(self):
        return {
            # XXX: Pick a better prefix. Maybe install to C:\<project-name>?
            InstallRoot.prefix:     Path('C:\\', Root.absolute),
            InstallRoot.bindir:     Path('', InstallRoot.prefix),
            InstallRoot.libdir:     Path('', InstallRoot.prefix),
            InstallRoot.includedir: Path('', InstallRoot.prefix),
        }


class CygwinPlatform(WindowsPlatform):
    @property
    def flavor(self):
        return 'posix'


def platform_info(name=None):
    if name is None:
        name = platform_name()

    if name == 'windows':
        return WindowsPlatform(name)
    elif name == 'cygwin':
        return CygwinPlatform(name)
    elif name == 'darwin':
        return DarwinPlatform(name)
    elif name == 'linux':
        return LinuxPlatform(name)
    else:  # Probably some POSIX system
        return PosixPlatform(name)


def which(names, env=os.environ):
    paths = env.get('PATH', os.defpath).split(os.pathsep)
    if platform_name() in ['windows', 'cygwin']:
        exts = env.get('PATHEXT', '').split(os.pathsep)
    else:
        exts = ['']

    for name in iterate(names):
        if os.path.isabs(name):
            if os.path.exists(name):
                return name
        else:
            for path in ['.'] if os.path.dirname(name) else paths:
                for ext in exts:
                    fullpath = os.path.normpath(os.path.join(path, name + ext))
                    if os.path.exists(fullpath):
                        return fullpath

    raise IOError("unable to find executable '{}'".format(name))



import os.path

from .integration import *
pjoin = os.path.join


class TestInstall(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'install', dist=True, *args, **kwargs)

    def setUp(self):
        IntegrationTest.setUp(self)
        cleandir(self.distdir)

    def test_default(self):
        self.build()
        self.assertOutput([executable('program')], 'hello, library!\n')

    @skip_if_backend('msbuild')
    @xfail_if_platform('darwin')
    def test_install(self):
        self.build('install')

        self.assertExists(pjoin(self.includedir, 'library.hpp'))
        self.assertExists(pjoin(self.bindir, executable('program').path))
        self.assertExists(pjoin(self.libdir, shared_library('library').path))

        os.chdir(self.srcdir)
        cleandir(self.builddir)
        self.assertOutput([pjoin(self.bindir, executable('program').path)],
                          'hello, library!\n')

    @skip_if_backend('msbuild')
    @xfail_if_platform('darwin')
    def test_install_existing_paths(self):
        makedirs(self.includedir, exist_ok=True)
        makedirs(self.bindir, exist_ok=True)
        makedirs(self.libdir, exist_ok=True)
        self.build('install')

        self.assertExists(pjoin(self.includedir, 'library.hpp'))
        self.assertExists(pjoin(self.bindir, executable('program').path))
        self.assertExists(pjoin(self.libdir, shared_library('library').path))

        os.chdir(self.srcdir)
        cleandir(self.builddir)
        self.assertOutput([pjoin(self.bindir, executable('program').path)],
                          'hello, library!\n')



import json
import os
from packaging.version import Version
from six import iteritems, string_types

from .backends import get_backends
from .path import Path, InstallRoot
from . import platforms
from . import tools


class EnvVersionError(RuntimeError):
    pass


class Environment(object):
    version = 6
    envfile = '.bfg_environ'

    def __new__(cls, *args, **kwargs):
        env = object.__new__(cls)
        env.__builders = {}
        env.__tools = {}
        return env

    def __init__(self, bfgpath, backend, backend_version, srcdir, builddir,
                 install_dirs):
        self.bfgpath = bfgpath
        self.backend = backend
        self.backend_version = backend_version

        self.srcdir = srcdir
        self.builddir = builddir
        self.install_dirs = install_dirs

        self.variables = dict(os.environ)
        self.platform = platforms.platform_info()

    def getvar(self, key, default=None):
        return self.variables.get(key, default)

    def builder(self, lang):
        # XXX: Be more intelligent about this when we support more languages.
        if not isinstance(lang, string_types):
            lang = 'c++' if 'c++' in lang else 'c'

        if lang not in self.__builders:
            self.__builders[lang] = tools.get_builder(lang, self)
        return self.__builders[lang]

    def compiler(self, lang):
        if not isinstance(lang, string_types):
            raise TypeError('only one lang supported')
        return self.builder(lang).compiler

    def linker(self, lang, mode):
        return self.builder(lang).linkers[mode]

    def tool(self, name):
        if name not in self.__tools:
            self.__tools[name] = tools.get_tool(name, self)
        return self.__tools[name]

    def save(self, path):
        with open(os.path.join(path, self.envfile), 'w') as out:
            json.dump({
                'version': self.version,
                'data': {
                    'bfgpath': self.bfgpath.to_json(),
                    'platform': self.platform.name,
                    'backend': self.backend,
                    'backend_version': str(self.backend_version),
                    'variables': self.variables,
                    'srcdir': self.srcdir.to_json(),
                    'builddir': self.builddir.to_json(),
                    'install_dirs': {
                        k.name: v.to_json() for k, v in
                        iteritems(self.install_dirs)
                    },
                }
            }, out)

    @classmethod
    def load(cls, path):
        with open(os.path.join(path, cls.envfile)) as inp:
            state = json.load(inp)
            version, data = state['version'], state['data']
        if version > cls.version:
            raise EnvVersionError('saved version exceeds expected version')

        env = Environment.__new__(Environment)

        for i in ['backend', 'variables']:
            setattr(env, i, data[i])

        if version <= 5:
            backend_version = get_backends()[data['backend']].version()
        else:
            backend_version = Version(data['backend_version'])
        setattr(env, 'backend_version', backend_version)

        for i in ['bfgpath', 'srcdir', 'builddir']:
            if version <= 4 or (version <= 5 and i == 'bfgpath'):
                setattr(env, i, Path(data[i]))
            else:
                setattr(env, i, Path.from_json(data[i]))

        env.platform = platforms.platform_info(data['platform'])
        env.install_dirs = {
            InstallRoot[k]: Path.from_json(v) for k, v in
            iteritems(data['install_dirs'])
        }

        return env



from .integration import *


class TestExplicitDefaults(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'explicit_defaults', *args, **kwargs)

    @skip_if_backend('msbuild')
    def test_default(self):
        self.build()
        self.assertOutput([executable('a')], 'hello, a!\n')
        self.assertNotExists(executable('b'))


class TestImplicitDefaults(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'implicit_defaults', *args, **kwargs)

    @skip_if_backend('msbuild')
    def test_default(self):
        self.build()
        self.assertOutput([executable('a')], 'hello, a!\n')
        self.assertOutput([executable('b')], 'hello, b!\n')
        self.assertNotExists(executable('test'))



import unittest

from bfg9000.shell.windows import *


class TestSplit(unittest.TestCase):
    def test_single(self):
        self.assertEqual(split('foo'), ['foo'])

    def test_multiple(self):
        self.assertEqual(split('foo bar baz'), ['foo', 'bar', 'baz'])

    def test_backslash(self):
        self.assertEqual(split(r'C:\path\to\file'), [r'C:\path\to\file'])

    def test_quote(self):
        self.assertEqual(split('foo "bar baz"'), ['foo', 'bar baz'])
        self.assertEqual(split('foo"bar baz"'), ['foobar baz'])
        self.assertEqual(split(r'foo "c:\path\\"'), ['foo', 'c:\\path\\'])
        self.assertEqual(split('foo "it\'s \\"good\\""'),
                         ['foo', 'it\'s "good"'])


class TestQuote(unittest.TestCase):
    def test_simple(self):
        self.assertEqual(quote('foo'), 'foo')

    def test_space(self):
        self.assertEqual(quote('foo bar'), '"foo bar"')

    def test_quote(self):
        self.assertEqual(quote('"foo"'), '"\\"foo\\""')

    def test_backslash(self):
        self.assertEqual(quote(r'foo\bar'), r'foo\bar')
        self.assertEqual(quote('foo\\bar\\'), r'"foo\bar\\"')

    def test_escaped_quote(self):
        self.assertEqual(quote(r'foo\"bar'), r'"foo\\\"bar"')



import os.path
from collections import defaultdict
from itertools import chain
from six import string_types

from .hooks import builtin
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..build_inputs import build_input, Edge
from ..file_types import *
from ..iterutils import iterate, listify, uniques
from ..path import Path, Root
from ..shell import posix as pshell


build_input('compile_options')(lambda: defaultdict(list))


class ObjectFiles(list):
    def __getitem__(self, key):
        if isinstance(key, string_types):
            key = Path(key, Root.srcdir)
        elif isinstance(key, File):
            key = key.path

        if isinstance(key, Path):
            for i in self:
                if i.creator and i.creator.file.path == key:
                    return i
            raise ValueError("{!r} not found".format(key))
        else:
            return list.__getitem__(self, key)


class Compile(Edge):
    def __init__(self, build, env, name, file, include=None,
                 packages=None, options=None, lang=None, extra_deps=None):
        if name is None:
            name = os.path.splitext(file)[0]

        self.file = sourcify(file, SourceFile, lang=lang)
        self.builder = env.compiler(self.file.lang)
        self.includes = [sourcify(i, HeaderDirectory)
                         for i in iterate(include)]
        self.packages = listify(packages)
        self.user_options = pshell.listify(options)
        self.link_options = []

        pkg_includes = chain.from_iterable(i.includes for i in self.packages)
        self.all_includes = uniques(chain(pkg_includes, self.includes))
        self._internal_options = sum(
            (self.builder.include_dir(i) for i in self.all_includes), []
        )

        target = self.builder.output_file(name)
        Edge.__init__(self, build, target, extra_deps)

    @property
    def options(self):
        return self._internal_options + self.link_options + self.user_options


@builtin.globals('build_inputs', 'env')
def object_file(build, env, name=None, file=None, **kwargs):
    if file is None:
        if name is None:
            raise TypeError('expected name')
        return ObjectFile(name, root=Root.srcdir, **kwargs)
    else:
        return Compile(build, env, name, file, **kwargs).target


@builtin.globals('build_inputs', 'env')
def object_files(build, env, files, **kwargs):
    def _compile(file, **kwargs):
        return Compile(build, env, None, file, **kwargs).target
    return ObjectFiles(objectify(i, ObjectFile, _compile, **kwargs)
                       for i in iterate(files))


@builtin.globals('build_inputs')
def global_options(build, options, lang):
    build['compile_options'][lang].extend(pshell.listify(options))


def _get_flags(backend, rule, build_inputs, buildfile):
    global_cflags, cflags = backend.flags_vars(
        rule.builder.command_var + 'flags',
        ( rule.builder.global_args +
          build_inputs['compile_options'][rule.file.lang] ),
        buildfile
    )

    variables = {}

    cflags_value = rule.options
    if cflags_value:
        variables[cflags] = [global_cflags] + cflags_value

    return variables, {'args': cflags}


@make.rule_handler(Compile)
def make_object_file(rule, build_inputs, buildfile, env):
    compiler = rule.builder
    variables, cmd_kwargs = _get_flags(make, rule, build_inputs, buildfile)

    recipename = make.var('RULE_{}'.format(compiler.rule_name.upper()))
    if not buildfile.has_variable(recipename):
        recipe_extra = []

        # Only GCC-style depfiles are supported by Make.
        if compiler.deps_flavor == 'gcc':
            depfixer = env.tool('depfixer')
            cmd_kwargs['deps'] = deps = make.qvar('@') + '.d'
            df_cmd = make.cmd_var(depfixer, buildfile)
            recipe_extra = [make.silent(depfixer(df_cmd, deps))]

        buildfile.define(recipename, [compiler(
            cmd=make.cmd_var(compiler, buildfile), input=make.qvar('<'),
            output=make.qvar('@'), **cmd_kwargs
        )] + recipe_extra)

    path = rule.target.path
    out_dir = path.parent()
    buildfile.rule(
        target=path,
        deps=[rule.file] + rule.extra_deps,
        order_only=[out_dir.append(make.dir_sentinel)] if out_dir else None,
        recipe=recipename,
        variables=variables
    )
    buildfile.include(path.addext('.d'), optional=True)


@ninja.rule_handler(Compile)
def ninja_object_file(rule, build_inputs, buildfile, env):
    compiler = rule.builder
    variables, cmd_kwargs = _get_flags(ninja, rule, build_inputs, buildfile)

    if not buildfile.has_rule(compiler.rule_name):
        depfile = None
        deps = None

        if compiler.deps_flavor == 'gcc':
            deps = 'gcc'
            cmd_kwargs['deps'] = depfile = ninja.var('out') + '.d'
        elif compiler.deps_flavor == 'msvc':
            deps = 'msvc'
            cmd_kwargs['deps'] = True

        buildfile.rule(name=compiler.rule_name, command=compiler(
            cmd=ninja.cmd_var(compiler, buildfile), input=ninja.var('in'),
            output=ninja.var('out'), **cmd_kwargs
        ), depfile=depfile, deps=deps)

    buildfile.build(
        output=rule.target,
        rule=compiler.rule_name,
        inputs=[rule.file],
        implicit=rule.extra_deps,
        variables=variables
    )



from six import string_types

from . import iterutils


class safe_string(object):
    pass


def safe_str(s):
    if isinstance(s, string_types) or isinstance(s, safe_string):
        return s
    elif hasattr(s, '_safe_str'):
        return s._safe_str()
    else:
        raise NotImplementedError(type(s))


class escaped_str(safe_string):
    def __init__(self, string):
        if not isinstance(string, string_types):
            raise TypeError('expected a string')
        self.string = string

    def __str__(self):
        raise NotImplementedError()

    def __repr__(self):
        return '`{}`'.format(self.string)

    def __eq__(self, rhs):
        if not isinstance(rhs, escaped_str):
            return NotImplemented
        return self.string == rhs.string

    def __add__(self, rhs):
        return jbos(self, rhs)

    def __radd__(self, lhs):
        return jbos(lhs, self)


class jbos(safe_string):  # Just a Bunch of Strings
    def __init__(self, *args):
        self.__bits = tuple(self.__flatten(args))

    @staticmethod
    def __flatten(value):
        for i in value:
            if isinstance(i, jbos):
                for j in i.bits:
                    yield j
            elif isinstance(i, string_types) or isinstance(i, safe_string):
                yield i
            else:
                raise TypeError(type(i))

    @property
    def bits(self):
        return self.__bits

    def __str__(self):
        raise NotImplementedError()

    def __repr__(self):
        return 'jbos({})'.format(', '.join(repr(i) for i in self.bits))

    def __add__(self, rhs):
        return jbos(self, rhs)

    def __radd__(self, lhs):
        return jbos(lhs, self)


def join(iterable, delim):
    return sum(iterutils.tween(iterable, delim), jbos())



import platform
from packaging.specifiers import SpecifierSet
from packaging.version import Version

from .hooks import builtin
from ..version import version as _bfg_version
from ..file_types import objectify

_bfg_version = Version(_bfg_version)
_python_version = Version(platform.python_version())


def make_specifier(s, prereleases=None):
    if s is None:
        return None
    return objectify(s, SpecifierSet, None, prereleases=prereleases)


def check_version(version, specifier, kind):
    msg = "{kind} version {ver} doesn't meet requirement {req}"
    if specifier and version not in specifier:
        raise ValueError(msg.format(kind=kind, ver=version, req=specifier))


@builtin
def bfg9000_required_version(version=None, python_version=None):
    version = make_specifier(version, prereleases=True)
    python_version = make_specifier(python_version, prereleases=True)

    check_version(_bfg_version, version, kind='bfg9000')
    check_version(_python_version, python_version, kind='python')



from .iterutils import iterate

ext2lang = {}


def lang_exts(lang, exts):
    for i in iterate(exts):
        if i in ext2lang:
            raise ValueError('{ext} already used by {lang}'.format(
                ext=i, lang=lang
            ))
        ext2lang[i] = lang



import os.path

from .integration import *


class TestFindFiles(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(
            self, os.path.join(examples_dir, '06_find_files'), *args, **kwargs
        )

    def test_hello(self):
        self.build(executable('hello'))
        self.assertOutput([executable('hello')], 'Hello, world!\n')

    def test_goodbye(self):
        self.build(executable('goodbye'))
        self.assertOutput([executable('goodbye')],
                          'Goodbye!\nAuf Wiedersehen!\n')



from .integration import *


class TestCompilerFlavor(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'compiler_flavor', *args, **kwargs)

    def test_build(self):
        self.build(executable('program'))
        self.assertOutput(
            [executable('program')],
            'hello, {}!\n'.format(env.compiler('c++').flavor)
        )



from .hooks import builtin
from ..file_types import *


@builtin
def source_file(name, lang=None):
    # XXX: Add a way to make a generic File object instead of a SourceFile?
    return SourceFile(name, Root.srcdir, lang)


@builtin
def directory(name):
    return Directory(name, Root.srcdir)


@builtin
def header(name):
    return HeaderFile(name, Root.srcdir)


@builtin
def header_directory(name, system=False):
    return HeaderDirectory(name, Root.srcdir, system)



import os.path
from six import string_types

from .languages import ext2lang
from .path import InstallRoot, Path, Root
from .safe_str import safe_str


class Node(object):
    def __init__(self):
        self.creator = None

    @property
    def all(self):
        return [self]

    def _safe_str(self):
        return safe_str(self.path)

    def __repr__(self):
        return '<{type} {name}>'.format(
            type=type(self).__name__, name=repr(self.path)
        )


def objectify(thing, valid_type, creator, *args, **kwargs):
    if isinstance(thing, valid_type):
        return thing
    elif not isinstance(thing, string_types):
        raise TypeError('expected a {} or a string'.format(valid_type))
    else:
        if creator is None:
            creator = valid_type
        # XXX: Come up with a way to provide args to prepend?
        return creator(thing, *args, **kwargs)


def sourcify(thing, valid_type, make_type=None, **kwargs):
    return objectify(thing, valid_type, make_type, root=Root.srcdir, **kwargs)


class File(Node):
    install_kind = None
    install_root = None

    def __init__(self, name, root):
        Node.__init__(self)
        self.path = Path(name, root)
        self.post_install = None


class Directory(File):
    pass


class Phony(Node):
    def __init__(self, name):
        Node.__init__(self)
        self.path = name


class SourceFile(File):
    def __init__(self, name, root, lang=None):
        File.__init__(self, name, root)
        if lang is None:
            lang = ext2lang.get(os.path.splitext(name)[1])
        self.lang = lang


class HeaderFile(File):
    install_kind = 'data'
    install_root = InstallRoot.includedir


class HeaderDirectory(Directory):
    install_root = InstallRoot.includedir

    def __init__(self, name, root, system=False):
        Directory.__init__(self, name, root)
        self.system = system


class ObjectFile(File):
    def __init__(self, name, root, lang):
        File.__init__(self, name, root)
        self.lang = lang


class Binary(File):
    install_kind = 'program'

    def __init__(self, name, root, lang=None):
        File.__init__(self, name, root)
        self.lang = lang


class Executable(Binary):
    install_root = InstallRoot.bindir


class SystemExecutable(Executable):
    install_root = None


class Library(Binary):
    install_root = InstallRoot.libdir

    @property
    def link(self):
        return self


class StaticLibrary(Library):
    pass


class WholeArchive(StaticLibrary):
    def __init__(self, lib):
        Node.__init__(self)
        self.lib = lib
        self.creator = lib.creator
        self.post_install = None

    @property
    def path(self):
        return self.lib.path

    @property
    def lang(self):
        return self.lib.lang


class SharedLibrary(Library):
    pass


class ImportLibrary(SharedLibrary):
    pass


class DllLibrary(SharedLibrary):
    install_root = InstallRoot.bindir

    def __init__(self, name, import_name, root, lang):
        SharedLibrary.__init__(self, name, root, lang)
        self.import_lib = ImportLibrary(import_name, root, lang)

    @property
    def all(self):
        return [self, self.import_lib]

    @property
    def link(self):
        return self.import_lib



from .hooks import builtin
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..build_inputs import build_input


@build_input('defaults')
class DefaultTargets(object):
    def __init__(self):
        self.default_targets = []
        self.fallback_defaults = []

    def add(self, target, explicit=False):
        targets = self.default_targets if explicit else self.fallback_defaults
        targets.append(target)

    def remove(self, target):
        for i, fallback in enumerate(self.fallback_defaults):
            if target is fallback:
                self.fallback_defaults.pop(i)

    @property
    def targets(self):
        return self.default_targets or self.fallback_defaults


@builtin.globals('build_inputs')
def default(build, *args):
    for i in args:
        if i.creator:
            build['defaults'].add(i, explicit=True)


@make.pre_rule
def make_all_rule(build_inputs, buildfile, env):
    buildfile.rule(
        target='all',
        deps=build_inputs['defaults'].targets,
        phony=True
    )


@ninja.pre_rule
def ninja_all_rule(build_inputs, buildfile, env):
    buildfile.default(['all'])
    buildfile.build(
        output='all',
        rule='phony',
        inputs=build_inputs['defaults'].targets,
    )



from .hooks import tool
from ..platforms import platform_name
from ..safe_str import escaped_str, jbos, safe_str


@tool('bfg9000')
class Bfg9000(object):
    rule_name = command_var = 'bfg9000'

    def __init__(self, env):
        self.command = env.getvar('BFG9000', env.bfgpath)

    def regenerate(self, cmd, builddir):
        return [cmd, '--regenerate', builddir]


@tool('depfixer')
class Depfixer(object):
    rule_name = command_var = 'depfixer'

    def __init__(self, env):
        default = env.bfgpath.parent().append('bfg9000-depfixer')
        self.command = env.getvar('DEPFIXER', default)

    def __call__(self, cmd, depfile):
        return cmd + ' < ' + depfile + ' >> ' + depfile


if platform_name() == 'windows':
    @tool('setenv')
    class SetEnv(object):
        rule_name = command_var = 'setenv'

        def __init__(self, env):
            default = env.bfgpath.parent().append('bfg9000-setenv')
            self.command = env.getvar('SETENV', default)

        def __call__(self, cmd, env):
            if env:
                eq = escaped_str('=')
                return [cmd] + [jbos(safe_str(name), eq, safe_str(value))
                                for name, value in env.iteritems()] + ['--']
            return []



from itertools import chain
from six.moves import cStringIO as StringIO

from .hooks import builtin
from .. import safe_str
from .. import shell
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..build_inputs import Edge
from ..file_types import Phony
from ..shell import posix as pshell


class Command(Edge):
    def __init__(self, build, name, cmd=None, cmds=None, environment=None,
                 extra_deps=None):
        if (cmd is None) == (cmds is None):
            raise ValueError('exactly one of "cmd" or "cmds" must be ' +
                             'specified')
        elif cmds is None:
            cmds = [cmd]

        self.cmds = cmds
        self.env = environment or {}
        Edge.__init__(self, build, Phony(name), extra_deps)


@builtin.globals('build_inputs')
def command(build, *args, **kwargs):
    return Command(build, *args, **kwargs).target


@make.rule_handler(Command)
def make_command(rule, build_inputs, buildfile, env):
    # Join all the commands onto one line so that users can use 'cd' and such.
    out = make.Writer(StringIO())
    env_vars = pshell.global_env(rule.env)

    for line in shell.join_commands(chain(env_vars, rule.cmds)):
        out.write_shell(line)

    buildfile.rule(
        target=rule.target,
        deps=rule.extra_deps,
        recipe=[safe_str.escaped_str(out.stream.getvalue())],
        phony=True
    )


@ninja.rule_handler(Command)
def ninja_command(rule, build_inputs, buildfile, env):
    ninja.command_build(
        buildfile, env,
        output=rule.target,
        inputs=rule.extra_deps,
        commands=rule.cmds,
        environ=rule.env
    )



from .hooks import tool


@tool('install')
class Install(object):
    rule_name = command_var = 'install'

    def __init__(self, env):
        default = 'ginstall' if env.platform.name == 'darwin' else 'install'
        self.command = env.getvar('INSTALL', default)

    @property
    def data_args(self):
        return ['-m', '644']

    def __call__(self, cmd, src, dst):
        return [cmd, '-D', src, dst]



import os.path
from itertools import chain

from .utils import library_macro
from .. import shell
from ..file_types import *
from ..iterutils import iterate, uniques
from ..path import Root


class MsvcCompiler(object):
    def __init__(self, env, lang, name, command, cflags):
        self.platform = env.platform
        self.lang = lang

        self.rule_name = self.command_var = name
        self.command = command

        self.global_args = ['/nologo'] + cflags

    @property
    def flavor(self):
        return 'msvc'

    @property
    def deps_flavor(self):
        return 'msvc'

    def __call__(self, cmd, input, output, deps=None, args=None):
        result = [cmd]
        result.extend(iterate(args))
        if deps:
            result.append('/showIncludes')
        result.extend(['/c', input])
        result.append('/Fo' + output)
        return result

    def output_file(self, name):
        return ObjectFile(name + '.obj', Root.builddir, self.lang)

    @property
    def library_args(self):
        return []

    def include_dir(self, directory):
        return ['/I' + directory.path]

    def link_args(self, name, mode):
        if mode == 'executable':
            return []
        elif mode in ['shared_library', 'static_library']:
            return ['/D' + library_macro(name, mode)]
        else:
            raise ValueError("unknown mode '{}'".format(mode))


class MsvcLinker(object):
    def __init__(self, env, mode, lang, name, command, ldflags, ldlibs):
        self.platform = env.platform
        self.mode = mode
        self.lang = lang

        self.rule_name = self.command_var = 'link_' + name
        self.command = command
        self.link_var = 'ld'

        self.global_args = ['/nologo'] + ldflags
        self.global_libs = ldlibs

    @property
    def flavor(self):
        return 'msvc'

    def __call__(self, cmd, input, output, libs=None, args=None):
        result = [cmd]
        result.extend(iterate(args))
        result.extend(iterate(input))
        result.extend(iterate(libs))
        result.append('/OUT:' + output)
        return result

    def output_file(self, name):
        if self.mode == 'executable':
            return Executable(name + self.platform.executable_ext,
                              Root.builddir, self.lang)
        elif self.mode == 'shared_library':
            return DllLibrary(name + self.platform.shared_library_ext,
                              name + '.lib', Root.builddir, self.lang)
        else:
            raise ValueError("unknown mode '{}'".format(self.mode))

    @property
    def auto_link(self):
        return True

    @property
    def mode_args(self):
        return ['/DLL'] if self.mode == 'shared_library' else []

    def lib_dirs(self, libraries, target):
        def get_dir(lib):
            return lib.path.parent() if isinstance(lib, Library) else lib
        dirs = uniques(get_dir(i) for i in iterate(libraries))
        return ['/LIBPATH:' + i for i in dirs]

    def link_lib(self, library):
        if isinstance(library, WholeArchive):
            raise ValueError('MSVC does not support whole-archives')
        return [library.link.path.basename()]

    def import_lib(self, library):
        if self.mode != 'shared_library':
            return []
        return ['/IMPLIB:' + library.import_lib.path]


class MsvcStaticLinker(object):
    def __init__(self, env, lang, name, command):
        self.platform = env.platform
        self.mode = 'static_library'
        self.lang = lang

        self.rule_name = self.command_var = 'lib_' + name
        self.command = command
        self.link_var = 'lib'

        self.global_args = shell.split(env.getvar('LIBFLAGS', ''))

    @property
    def flavor(self):
        return 'msvc'

    def __call__(self, cmd, input, output, args=None):
        result = [cmd]
        result.extend(iterate(args))
        result.extend(iterate(input))
        result.append('/OUT:' + output)
        return result

    def output_file(self, name):
        return StaticLibrary(name + '.lib', Root.builddir, self.lang)

    @property
    def mode_args(self):
        return []


class MsvcPackageResolver(object):
    def __init__(self, env, lang):
        value = env.getvar('CPATH')
        user_include_dirs = value.split(os.pathsep) if value else []

        value = env.getvar('INCLUDE')
        system_include_dirs = value.split(os.pathsep) if value else []

        self.include_dirs = [i for i in uniques(chain(
            user_include_dirs, system_include_dirs, env.platform.include_dirs
        )) if os.path.exists(i)]

        value = env.getvar('LIB')
        system_lib_dirs = value.split(os.pathsep) if value else []

        value = env.getvar('LIBRARY_PATH')
        user_lib_dirs = value.split(os.pathsep) if value else []

        all_lib_dirs = ( os.path.abspath(i) for i in
                         chain(user_lib_dirs, system_lib_dirs) )
        self.lib_dirs = [i for i in uniques(chain(
            all_lib_dirs, env.platform.lib_dirs
        )) if os.path.exists(i)]

        self.lang = lang

    def header(self, name, search_dirs=None):
        if search_dirs is None:
            search_dirs = self.include_dirs

        for base in search_dirs:
            if os.path.exists(os.path.join(base, name)):
                return HeaderDirectory(base, Root.absolute, system=True)

        raise ValueError("unable to find header '{}'".format(name))

    def library(self, name, kind='any', search_dirs=None):
        if search_dirs is None:
            search_dirs = self.lib_dirs
        libname = name + '.lib'

        for base in search_dirs:
            fullpath = os.path.join(base, libname)
            if os.path.exists(fullpath):
                # We don't actually know what kind of library this is. It could
                # be a static library or an import library (which we classify
                # as a kind of shared lib).
                return Library(fullpath, Root.absolute, self.lang)
        raise ValueError("unable to find library '{}'".format(name))



import re
from collections import namedtuple, OrderedDict
from enum import Enum
from packaging.version import Version
from six import iteritems, string_types
from six.moves import cStringIO as StringIO

from ... import path
from ... import safe_str
from ... import shell
from ... import iterutils

__all__ = ['NinjaFile', 'Section', 'Syntax', 'Writer', 'Variable', 'var',
           'path_vars']

Rule = namedtuple('Rule', ['command', 'depfile', 'deps', 'generator', 'pool',
                           'restat'])
Build = namedtuple('Build', ['outputs', 'rule', 'inputs', 'implicit',
                             'order_only', 'variables'])

Syntax = Enum('Syntax', ['output', 'input', 'shell', 'clean'])
Section = Enum('Section', ['path', 'command', 'flags', 'other'])


class Writer(object):
    def __init__(self, stream):
        self.stream = stream

    @staticmethod
    def escape_str(string, syntax):
        if '\n' in string:
            raise ValueError('illegal newline')

        if syntax == Syntax.output:
            return re.sub(r'([:$ ])', r'$\1', string)
        elif syntax == Syntax.input:
            return re.sub(r'([$ ])', r'$\1', string)
        elif syntax in [Syntax.shell, Syntax.clean]:
            return string.replace('$', '$$')
        else:
            raise ValueError("unknown syntax '{}'".format(syntax))

    def write_literal(self, string):
        self.stream.write(string)

    def write(self, thing, syntax, shell_quote=shell.quote_info):
        thing = safe_str.safe_str(thing)
        shelly = syntax == Syntax.shell
        escaped = False

        if isinstance(thing, safe_str.escaped_str):
            self.write_literal(thing.string)
            escaped = True
        elif isinstance(thing, string_types):
            if shelly and shell_quote:
                thing, escaped = shell_quote(thing)
            self.write_literal(self.escape_str(thing, syntax))
        elif isinstance(thing, safe_str.jbos):
            for i in thing.bits:
                escaped |= self.write(i, syntax, shell_quote)
        elif isinstance(thing, path.Path):
            out = Writer(StringIO())
            thing = thing.realize(path_vars, shelly)
            escaped = out.write(thing, syntax, shell.escape)

            thing = out.stream.getvalue()
            if shelly and escaped:
                thing = shell.quote_escaped(thing)
            self.write_literal(thing)
        else:
            raise TypeError(type(thing))

        return escaped

    def write_each(self, things, syntax, delim=safe_str.escaped_str(' '),
                   prefix=None, suffix=None):
        for i in iterutils.tween(things, delim, prefix, suffix):
            self.write(i, syntax)

    def write_shell(self, thing, clean=False):
        syntax = Syntax.clean if clean else Syntax.shell
        if iterutils.isiterable(thing):
            self.write_each(thing, syntax)
        else:
            self.write(thing, syntax, shell_quote=None)


class Variable(object):
    def __init__(self, name):
        self.name = re.sub('\W', '_', name)

    def use(self):
        return safe_str.escaped_str('${}'.format(self.name))

    def _safe_str(self):
        return self.use()

    def __str__(self):
        raise NotImplementedError()

    def __repr__(self):
        return repr(self.use())

    def __hash__(self):
        return hash(self.name)

    def __eq__(self, rhs):
        return self.name == rhs.name

    def __add__(self, rhs):
        return self.use() + rhs

    def __radd__(self, lhs):
        return lhs + self.use()


def var(v):
    return v if isinstance(v, Variable) else Variable(v)


path_vars = {
    path.Root.srcdir:   Variable('srcdir'),
    path.Root.builddir: None,
}
path_vars.update({i: Variable(i.name) for i in path.InstallRoot})


class NinjaFile(object):
    def __init__(self):
        self._min_version = None
        self._var_table = set()
        self._variables = {i: [] for i in Section}

        self._rules = OrderedDict()

        self._builds = []
        self._build_outputs = set()
        self._defaults = []

    def min_version(self, version):
        if ( self._min_version is None or
             Version(version) > Version(self._min_version) ):
            self._min_version = version

    def variable(self, name, value, section=Section.other, exist_ok=True):
        name = var(name)
        if self.has_variable(name):
            if not exist_ok:
                raise ValueError("variable {!r} already exists".format(name))
        else:
            self._var_table.add(name)
            self._variables[section].append((name, value))
        return name

    def has_variable(self, name):
        return var(name) in self._var_table

    def rule(self, name, command, depfile=None, deps=None, generator=False,
             pool=None, restat=False):
        if pool is not None:
            if pool == 'console':
                self.min_version('1.5')
            else:
                raise ValueError("unknown pool '{}'".format(pool))
        if re.search('\W', name):
            raise ValueError('rule name contains invalid characters')
        if self.has_rule(name):
            raise ValueError("rule '{}' already exists".format(name))
        self._rules[name] = Rule(command, depfile, deps, generator, pool,
                                 restat)

    def has_rule(self, name):
        return name in self._rules

    def build(self, output, rule, inputs=None, implicit=None, order_only=None,
              variables=None):
        if rule != 'phony' and not self.has_rule(rule):
            raise ValueError("unknown rule '{}'".format(rule))

        variables = {var(k): v for k, v in iteritems(variables or {})}

        outputs = iterutils.listify(output)
        for i in outputs:
            if self.has_build(i):
                raise ValueError("build for '{}' already exists".format(i))
            self._build_outputs.add(i)
        self._builds.append(Build(
            outputs, rule, iterutils.listify(inputs),
            iterutils.listify(implicit), iterutils.listify(order_only),
            variables
        ))

    def has_build(self, name):
        return name in self._build_outputs

    def default(self, paths):
        self._defaults.extend(paths)

    def _write_variable(self, out, name, value, clean=False, indent=0):
        out.write_literal(('  ' * indent) + name.name + ' = ')
        out.write_shell(value, clean)
        out.write_literal('\n')

    def _write_rule(self, out, name, rule):
        out.write_literal('rule ' + name + '\n')

        self._write_variable(out, var('command'), rule.command, indent=1)
        if rule.depfile:
            self._write_variable(out, var('depfile'), rule.depfile, indent=1)
        if rule.deps:
            self._write_variable(out, var('deps'), rule.deps, indent=1)
        if rule.generator:
            self._write_variable(out, var('generator'), '1', indent=1)
        if rule.pool:
            self._write_variable(out, var('pool'), rule.pool, indent=1)
        if rule.restat:
            self._write_variable(out, var('restat'), '1', indent=1)

    def _write_build(self, out, build):
        out.write_literal('build ')
        out.write_each(build.outputs, Syntax.output)
        out.write_literal(': ' + build.rule)

        esc = safe_str.escaped_str
        out.write_each(build.inputs, Syntax.input, prefix=esc(' '))
        out.write_each(build.implicit, Syntax.input, prefix=esc(' | '))
        out.write_each(build.order_only, Syntax.input, prefix=esc(' || '))
        out.write_literal('\n')

        if build.variables:
            for k, v in iteritems(build.variables):
                self._write_variable(out, k, v, indent=1)

    def write(self, out):
        out = Writer(out)

        if self._min_version:
            self._write_variable(
                out, var('ninja_required_version'), self._min_version
            )
            out.write_literal('\n')

        for section in Section:
            # Paths are inherently clean (read: don't need shell quoting).
            # XXX: This behavior is a bit strange and maybe should be reworked.
            clean = section == Section.path
            for name, value in self._variables[section]:
                self._write_variable(out, name, value, clean)
            if self._variables[section]:
                out.write_literal('\n')

        for name, rule in iteritems(self._rules):
            self._write_rule(out, name, rule)
            out.write_literal('\n')

        for build in self._builds:
            self._write_build(out, build)

        if self._defaults:
            out.write_literal('\ndefault ')
            out.write_each(self._defaults, Syntax.input)
            out.write_literal('\n')



from functools import partial
from six.moves import cStringIO as StringIO

from .hooks import builtin
from .packages import SystemExecutable
from .. import safe_str
from .. import shell
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..build_inputs import build_input
from ..file_types import Executable, File, objectify, sourcify
from ..shell import posix as pshell


@build_input('tests')
class TestInputs(object):
    def __init__(self):
        self.tests = []
        self.targets = []
        self.extra_deps = []

    def __nonzero__(self):
        return bool(self.tests)


class TestCase(object):
    def __init__(self, target, options, env):
        self.target = target
        self.options = options
        self.env = env


class TestDriver(object):
    def __init__(self, target, options, env):
        self.target = target
        self.options = options
        self.env = env
        self.tests = []


@builtin.globals('build_inputs')
def test(build, test, options=None, environment=None, driver=None):
    if driver and environment:
        raise TypeError('only one of "driver" and "environment" may be ' +
                        'specified')

    test = sourcify(test, File)
    build['tests'].targets.append(test)
    build['defaults'].remove(test)

    case = TestCase(test, pshell.listify(options), environment or {})
    (driver or build['tests']).tests.append(case)
    return case


@builtin.globals('builtins', 'build_inputs', 'env')
def test_driver(builtins, build, env, driver, options=None, environment=None,
                parent=None):
    if parent and environment:
        raise TypeError('only one of "parent" and "environment" may be ' +
                        'specified')

    driver = objectify(driver, Executable, builtins['system_executable'])
    result = TestDriver(driver, pshell.listify(options), environment or {})
    (parent or build['tests']).tests.append(result)
    return result


@builtin.globals('build_inputs')
def test_deps(build, *args):
    if len(args) == 0:
        raise ValueError('expected at least one argument')
    build['tests'].extra_deps.extend(args)


def _build_commands(tests, writer, local_env, collapse=False):
    def command(test, args=None):
        env_vars = local_env(test.env)
        subcmd = env_vars + [test.target] + test.options + (args or [])

        if collapse:
            out = writer(StringIO())
            out.write_shell(subcmd)
            s = out.stream.getvalue()
            if len(subcmd) > 1:
                s = shell.quote(s)
            return safe_str.escaped_str(s)
        return subcmd

    cmd, deps = [], []
    for i in tests:
        if type(i) == TestDriver:
            args, moredeps = _build_commands(i.tests, writer, local_env, True)
            if type(i.target) != SystemExecutable:
                deps.append(i.target)
            deps.extend(moredeps)
            cmd.append(command(i, args))
        else:
            cmd.append(command(i))
    return cmd, deps


@make.post_rule
def make_test_rule(build_inputs, buildfile, env):
    tests = build_inputs['tests']
    if not tests:
        return

    deps = []
    if tests.targets:
        buildfile.rule(
            target='tests',
            deps=tests.targets,
            phony=True
        )
        deps.append('tests')
    deps.extend(tests.extra_deps)

    try:
        local_env = shell.local_env
    except AttributeError:
        setenv = env.tool('setenv')
        local_env = partial(setenv, make.cmd_var(setenv, buildfile))

    recipe, moredeps = _build_commands(tests.tests, make.Writer, local_env)
    buildfile.rule(
        target='test',
        deps=deps + moredeps,
        recipe=recipe,
        phony=True
    )


@ninja.post_rule
def ninja_test_rule(build_inputs, buildfile, env):
    tests = build_inputs['tests']
    if not tests:
        return

    deps = []
    if tests.targets:
        buildfile.build(
            output='tests',
            rule='phony',
            inputs=tests.targets
        )
        deps.append('tests')
    deps.extend(tests.extra_deps)

    try:
        local_env = shell.local_env
    except AttributeError:
        setenv = env.tool('setenv')
        local_env = partial(setenv, ninja.cmd_var(setenv, buildfile))

    commands, moredeps = _build_commands(tests.tests, ninja.Writer, local_env)
    ninja.command_build(
        buildfile, env,
        output='test',
        inputs=deps + moredeps,
        commands=commands,
    )



import os.path
import re
from packaging.version import Version

from .hooks import builtin
from .find import find
from .version import check_version, make_specifier
from .. import path
from ..file_types import SystemExecutable
from ..iterutils import iterate, listify
from ..platforms import which


class Package(object):
    def __init__(self, includes=None, libraries=None, lib_dirs=None):
        self.includes = includes or []
        self.libraries = libraries or []
        self.lib_dirs = lib_dirs or []


class BoostPackage(Package):
    def __init__(self, includes=None, libraries=None, lib_dirs=None,
                 version=None):
        Package.__init__(self, includes, libraries, lib_dirs)
        self.version = version


def _boost_version(header, required_version=None):
    version_hpp = header.path.append('boost').append('version.hpp')
    with open(version_hpp.string()) as f:
        for line in f:
            m = re.match(r'#\s*define\s+BOOST_LIB_VERSION\s+"([\d_]+)"', line)
            if m:
                version = Version(m.group(1).replace('_', '.'))
                check_version(version, required_version, 'Boost')
                return version
    raise IOError('unable to parse "boost/version.hpp"')


@builtin.globals('env')
def system_package(env, name, lang='c', kind='any'):
    if kind not in ('any', 'shared', 'static'):
        raise ValueError("kind must be one of 'any', 'shared', or 'static'")
    return Package([], [env.builder(lang).packages.library(name, kind)])


@builtin.globals('env')
def boost_package(env, name=None, version=None):
    version = make_specifier(version)
    pack = env.builder('c++').packages
    version_hpp = os.path.join('boost', 'version.hpp')

    root = env.getvar('BOOST_ROOT')
    incdir = env.getvar('BOOST_INCLUDEDIR', os.path.join(root, 'include')
                        if root else None)
    libdir = env.getvar('BOOST_LIBRARYDIR', os.path.join(root, 'lib')
                        if root else None)

    if incdir:
        header = pack.header(version_hpp, [incdir])
        boost_version = _boost_version(header, version)
    else:
        # On Windows, check the default install location, which is structured
        # differently from other install locations.
        if env.platform.name == 'windows':
            dirs = find(r'C:\Boost\include', 'boost-*', type='d', flat=True)
            if dirs:
                try:
                    header = pack.header(version_hpp, max(dirs))
                    boost_version = _boost_version(header, version)
                    return BoostPackage(
                        includes=[header],
                        lib_dirs=r'C:\Boost\lib',
                        version=boost_version
                    )
                except IOError:
                    pass

        header = pack.header(version_hpp)
        boost_version = _boost_version(header, version)

    if env.platform.name == 'windows':
        if not env.linker('c++', 'shared_library').auto_link:
            # XXX: Don't require auto-link.
            raise ValueError('Boost on Windows requires auto-link')
        return BoostPackage(
            includes=[header],
            lib_dirs=listify(libdir),
            version=boost_version
        )
    else:
        dirs = [libdir] if libdir else None
        return BoostPackage(
            includes=[header],
            libraries=[ pack.library('boost_' + i, search_dirs=dirs)
                        for i in iterate(name) ],
            version=boost_version
        )


@builtin.globals('env')
def system_executable(env, name):
    return SystemExecutable(which(name, env.variables), path.Root.absolute)



import argparse
import sys

from .pathutils import makedirs
from .version import version


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('path', help='directory to create')
    parser.add_argument('--version', action='version',
                        version='%(prog)s ' + version)
    args = parser.parse_args()

    try:
        makedirs(args.path, exist_ok=True)
        return 0
    except Exception as e:
        sys.stderr.write('{prog}: error creating directories: {msg}\n'
                         .format(prog=parser.prog, msg=e))
        return 1



import functools
from six import iteritems

_all_builtins = {}


class Binder(object):
    def __init__(self, args, fn):
        self.__args = args
        self.__fn = fn

    def bind(self, **kwargs):
        # XXX: partial doesn't forward the docstring of the function.
        return functools.partial(self.__fn, *[kwargs[i] for i in self.__args])


def _decorate_builtin(*args):
    def wrapper(fn):
        bound = Binder(args, fn)
        _all_builtins[fn.__name__] = bound
        return bound
    return wrapper


builtin = _decorate_builtin()
builtin.globals = _decorate_builtin


def bind(**kwargs):
    builtins = {}
    for k, v in iteritems(_all_builtins):
        builtins[k] = v.bind(builtins=builtins, **kwargs)

    # XXX: Make this more generic?
    builtins['env'] = kwargs['env']
    return builtins



import os.path
import shutil

from .integration import *
pjoin = os.path.join


class TestRegenerate(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'regenerate', stage_src=True,
                                 *args, **kwargs)

    @skip_if_backend('msbuild')
    def test_build(self):
        self.build('foo')
        self.assertExists(pjoin(self.builddir, 'foo'))

    @skip_if_backend('msbuild')
    def test_regenerate(self):
        self.wait()
        with open(pjoin(self.srcdir, 'build.bfg'), 'a') as out:
            out.write("command('bar', cmd=['touch', 'bar'])\n")

        self.build('bar')
        self.assertExists(pjoin(self.builddir, 'bar'))


class TestRegenerateGlob(IntegrationTest):
    def __init__(self, *args, **kwargs):
        self.extradir = pjoin(test_data_dir, 'regenerate_glob')
        IntegrationTest.__init__(self, pjoin(examples_dir, '06_find_files'),
                                 stage_src=True, *args, **kwargs)

    def copyfile(self, src, dest=None):
        if dest is None:
            dest = src
        shutil.copy(pjoin(self.extradir, src),
                    pjoin(self.srcdir, dest))

    @skip_if_backend('msbuild')
    @skip_pred(lambda x: x.backend == 'make' and
               env.platform.name == 'windows', 'xfail on windows + make')
    def test_add_file(self):
        self.wait()
        self.copyfile(pjoin('src', 'hello', 'bonjour.hpp'))
        self.copyfile(pjoin('src', 'hello', 'bonjour.cpp'))
        self.copyfile(pjoin('src', 'hello', 'main_added.cpp'),
                      pjoin('src', 'hello', 'main.cpp'))

        self.build(executable('hello'))
        self.assertOutput([executable('hello')],
                          'Hello, world!\nBonjour le monde!\n')

    @skip_if_backend('msbuild')
    def test_add_dir(self):
        self.wait()
        shutil.copytree(pjoin(self.extradir, 'src', 'goodbye', 'french'),
                        pjoin(self.srcdir, 'src', 'goodbye', 'french'))
        self.copyfile(pjoin('src', 'goodbye', 'main_added.cpp'),
                      pjoin('src', 'goodbye', 'main.cpp'))

        self.build(executable('goodbye'))
        self.assertOutput([executable('goodbye')],
                          'Goodbye!\nAuf Wiedersehen!\nAu revoir!\n')

    @skip_if_backend('msbuild')
    @skip_pred(lambda x: x.backend == 'make' and
               env.platform.name == 'windows', 'xfail on windows + make')
    def test_remove_file(self):
        self.wait()
        os.unlink(pjoin(self.srcdir, 'src', 'hello', 'hello.cpp'))
        self.copyfile(pjoin('src', 'hello', 'main_removed.cpp'),
                      pjoin('src', 'hello', 'main.cpp'))
        self.build(executable('hello'))
        self.assertOutput([executable('hello')], '')

    @skip_if_backend('msbuild')
    def test_remove_dir(self):
        self.wait()
        cleandir(pjoin(self.srcdir, 'src', 'goodbye', 'german'),
                 recreate=False)
        self.copyfile(pjoin('src', 'goodbye', 'main_removed.cpp'),
                      pjoin('src', 'goodbye', 'main.cpp'))

        self.build(executable('goodbye'))
        self.assertOutput([executable('goodbye')], 'Goodbye!\n')



from __future__ import division

import re
from enum import Enum
from six import iteritems

from .. import iterutils
from ..safe_str import escaped_str, safe_str

__all__ = ['split', 'listify', 'escape', 'quote_escaped', 'quote',
           'quote_info', 'join_commands', 'global_env']

# XXX: We need a way to escape cmd.exe-specific characters.
_bad_chars = re.compile(r'(\s|"|\\$)')
_replace = re.compile(r'(\\*)("|$)')

_Token = Enum('Token', ['char', 'quote', 'space'])
_State = Enum('State', ['between', 'char', 'word', 'quoted'])


def _tokenize(s):
    escapes = 0
    for c in s:
        if c == '\\':
            escapes += 1
        elif c == '"':
            for i in range(escapes // 2):
                yield (_Token.char, type(s)('\\'))
            yield (_Token.char, '"') if escapes % 2 else (_Token.quote, None)
            escapes = 0
        else:
            for i in range(escapes):
                yield (_Token.char, type(s)('\\'))
            yield ((_Token.space if c in ' \t' else _Token.char), c)
            escapes = 0


def split(s):
    state = _State.between
    args = []

    for tok, value in _tokenize(s):
        if state == _State.between:
            if tok == _Token.char:
                args.append(value)
                state = _State.word
            elif tok == _Token.quote:
                args.append('')
                state = _State.quoted
        elif state == _State.word:
            if tok == _Token.quote:
                state = _State.quoted
            elif tok == _Token.char:
                args[-1] += value
            else:  # tok == _Token.space
                state = _State.between
        else:  # state == _State.quoted
            if tok == _Token.quote:
                state = _State.word
            else:
                args[-1] += value

    return args


def listify(thing):
    if thing is None:
        return []
    elif iterutils.isiterable(thing):
        return list(thing)
    else:
        return split(thing)


def escape(s):
    if not s:
        return '', False
    if not _bad_chars.search(s):
        return s, False

    def repl(m):
        quote = '\\' + m.group(2) if len(m.group(2)) else ''
        return m.group(1) * 2 + quote
    return _replace.sub(repl, s), True


def quote_escaped(s, escaped=True):
    return '"' + s + '"' if escaped else s


def quote(s):
    return quote_escaped(*escape(s))


def quote_info(s):
    s, esc = escape(s)
    return quote_escaped(s, esc), esc


def join_commands(commands):
    return iterutils.tween(commands, escaped_str(' && '))


def global_env(env):
    # Join the name and value so they get quoted together, if necessary.
    return [ ['set', safe_str(name) + '=' + safe_str(value)]
             for name, value in iteritems(env) ]



import argparse
import os
import subprocess
import sys

from .version import version


def main():
    parser = argparse.ArgumentParser(
        prog='bfg9000-setenv',
        usage='%(prog)s [OPTION]... [NAME=VALUE]... COMMAND [ARG]...',
        description='Set each NAME to VALUE in the environment and run ' +
                    'COMMAND.'
    )

    parser.add_argument('args', metavar='ARGS', nargs='*',
                        help='environment variable or command argument')
    parser.add_argument('--version', action='version',
                        version='%(prog)s ' + version)

    args = parser.parse_args().args
    for i, val in enumerate(args):
        name, sep, value = val.partition('=')
        if sep:
            os.environ[name] = value
        else:
            break

    if i == len(args):
        sys.stderr.write('{prog}: COMMAND is required\n'
                         .format(prog=parser.prog))
        return 1
    return subprocess.call(args[i:])



import os
import subprocess
from itertools import chain
from packaging.specifiers import SpecifierSet
from packaging.version import Version
from six.moves import cStringIO as StringIO

from ... import iterutils
from ... import path
from ... import safe_str
from ... import shell
from .syntax import *
from ...platforms import platform_name, which


def version(env=os.environ):
    try:
        ninja = which(env.get('NINJA', ['ninja', 'ninja-build']), env)
        output = subprocess.check_output(
            [ninja, '--version'],
            universal_newlines=True
        )
        return Version(output.strip())
    except IOError:
        return None

priority = 3

_rule_handlers = {}
_pre_rules = []
_post_rules = []


def rule_handler(rule_name):
    def decorator(fn):
        _rule_handlers[rule_name] = fn
        return fn
    return decorator


def pre_rule(fn):
    _pre_rules.append(fn)
    return fn


def post_rule(fn):
    _post_rules.append(fn)
    return fn


def write(env, build_inputs):
    buildfile = NinjaFile()
    buildfile.variable(path_vars[path.Root.srcdir], env.srcdir, Section.path)
    for i in path.InstallRoot:
        buildfile.variable(path_vars[i], env.install_dirs[i], Section.path)

    for i in _pre_rules:
        i(build_inputs, buildfile, env)
    for e in build_inputs.edges:
        _rule_handlers[type(e)](e, build_inputs, buildfile, env)
    for i in _post_rules:
        i(build_inputs, buildfile, env)

    with open(env.builddir.append('build.ninja').string(), 'w') as out:
        buildfile.write(out)


def cmd_var(cmd, buildfile):
    name = cmd.command_var
    return buildfile.variable(name, cmd.command, Section.command, True)


def flags_vars(name, value, buildfile):
    gflags = buildfile.variable('global_' + name, value, Section.flags, True)
    flags = buildfile.variable(name, gflags, Section.other, True)
    return gflags, flags


class Commands(object):
    def __init__(self, commands, environ=None):
        self.commands = iterutils.listify(commands)
        self.environ = environ or {}

    def use(self):
        out = Writer(StringIO())
        if self.__needs_shell and platform_name() == 'windows':
            out.write_literal('cmd /c ')

        env_vars = shell.global_env(self.environ)
        for line in shell.join_commands(chain(env_vars, self.commands)):
            out.write_shell(line)
        return safe_str.escaped_str(out.stream.getvalue())

    def _safe_str(self):
        return self.use()

    @property
    def __needs_shell(self):
        return (
            len(self.commands) + len(self.environ) > 1 or
            any(not iterutils.isiterable(i) for i in self.commands)
        )


def command_build(buildfile, env, output, inputs=None, implicit=None,
                  order_only=None, commands=None, environ=None):
    extra_kwargs = {}
    if env.backend_version in SpecifierSet('>=1.5'):
        extra_kwargs['pool'] = 'console'

    if not buildfile.has_rule('command'):
        buildfile.rule(name='command', command=var('cmd'), **extra_kwargs)
    if not buildfile.has_build('PHONY'):
        buildfile.build(output='PHONY', rule='phony')

    buildfile.build(
        output=output,
        rule='command',
        inputs=inputs,
        implicit=iterutils.listify(implicit) + ['PHONY'],
        order_only=order_only,
        variables={'cmd': Commands(commands, environ)}
    )



from .hooks import builtin
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..build_inputs import Edge
from ..file_types import Phony


class Alias(Edge):
    def __init__(self, build, name, deps=None):
        Edge.__init__(self, build, Phony(name), deps)


@builtin.globals('build_inputs')
def alias(build, *args, **kwargs):
    return Alias(build, *args, **kwargs).target


@make.rule_handler(Alias)
def make_alias(rule, build_inputs, buildfile, env):
    buildfile.rule(
        target=rule.target,
        deps=rule.extra_deps,
        phony=True
    )


@ninja.rule_handler(Alias)
def ninja_alias(rule, build_inputs, buildfile, env):
    buildfile.build(
        output=rule.target,
        rule='phony',
        inputs=rule.extra_deps
    )



import os

from .integration import *
pjoin = os.path.join


class TestSubdirs(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(
            self, pjoin(examples_dir, '05_subdirs'), dist=True, *args, **kwargs
        )

    def setUp(self):
        IntegrationTest.setUp(self)
        cleandir(self.distdir)

    def test_build(self):
        self.build()
        self.assertOutput([executable('sub/program')], 'hello, library!\n')

    @only_if_backend('make')
    def test_dir_sentinels(self):
        self.build()
        self.assertTrue(os.path.isfile('sub/.dir'))

    @skip_if_backend('msbuild')
    @xfail_if_platform('darwin')
    def test_install(self):
        self.build('install')

        self.assertExists(pjoin(self.includedir, 'library.hpp'))
        self.assertExists(pjoin(self.bindir, executable('sub/program').path))
        self.assertExists(pjoin(
            self.libdir, shared_library('sub/library').path
        ))

        os.chdir(self.srcdir)
        cleandir(self.builddir)
        self.assertOutput([pjoin(self.bindir, executable('sub/program').path)],
                          'hello, library!\n')

    @skip_if_backend('msbuild')
    @xfail_if_platform('darwin')
    def test_install_existing_paths(self):
        makedirs(self.includedir, exist_ok=True)
        makedirs(self.bindir, exist_ok=True)
        makedirs(self.libdir, exist_ok=True)
        self.build('install')

        self.assertExists(pjoin(self.includedir, 'library.hpp'))
        self.assertExists(pjoin(self.bindir, executable('sub/program').path))
        self.assertExists(pjoin(
            self.libdir, shared_library('sub/library').path
        ))

        os.chdir(self.srcdir)
        cleandir(self.builddir)
        self.assertOutput([pjoin(self.bindir, executable('sub/program').path)],
                          'hello, library!\n')



import re

_modes = {
    'shared_library': 'EXPORTS',
    'static_library': 'STATIC',
}


def library_macro(name, mode):
    # Since the name always begins with "lib", this always produces a valid
    # macro name.
    return '{name}_{suffix}'.format(
        name=re.sub(r'\W', '_', name.upper()), suffix=_modes[mode]
    )



import re
from shlex import shlex
from six import iteritems, string_types

from .. import iterutils
from ..safe_str import escaped_str, jbos, safe_str

__all__ = ['split', 'listify', 'escape', 'quote_escaped', 'quote',
           'quote_info', 'join_commands', 'local_env', 'global_env']

_bad_chars = re.compile(r'[^\w@%+:,./-]')


def split(s):
    if not isinstance(s, string_types):
        raise TypeError('expected a string')
    lexer = shlex(s, posix=True)
    lexer.commenters = ''
    lexer.whitespace_split = True
    return list(lexer)


def listify(thing):
    if thing is None:
        return []
    elif iterutils.isiterable(thing):
        return list(thing)
    else:
        return split(thing)


def escape(s):
    if not s:
        return '', False
    if not _bad_chars.search(s):
        return s, False
    return s.replace("'", "'\"'\"'"), True


def quote_escaped(s, escaped=True):
    return "'" + s + "'" if escaped else s


def quote(s):
    return quote_escaped(*escape(s))


def quote_info(s):
    s, esc = escape(s)
    return quote_escaped(s, esc), esc


def join_commands(commands):
    return iterutils.tween(commands, escaped_str(' && '))


def local_env(env):
    eq = escaped_str('=')
    return [ jbos(safe_str(name), eq, safe_str(value))
             for name, value in iteritems(env) ]


def global_env(env):
    eq = escaped_str('=')
    return [ ['export', jbos(safe_str(name), eq, safe_str(value))]
             for name, value in iteritems(env) ]



import unittest

from bfg9000.safe_str import *


class TestSafeStr(unittest.TestCase):
    def test_construct_from_strings(self):
        s = jbos('foo', 'bar', escaped_str('baz'))
        self.assertEqual(s.bits, ('foo', 'bar', escaped_str('baz')))

    def test_construct_from_jbos(self):
        s = jbos(jbos('foo', 'bar'), jbos(escaped_str('baz')))
        self.assertEqual(s.bits, ('foo', 'bar', escaped_str('baz')))

    def test_concatenate_from_strings(self):
        s = escaped_str('foo') + 'bar'
        self.assertEqual(s.bits, (escaped_str('foo'), 'bar'))

        s = 'foo' + escaped_str('bar')
        self.assertEqual(s.bits, ('foo', escaped_str('bar')))

        s = escaped_str('foo') + escaped_str('bar')
        self.assertEqual(s.bits, (escaped_str('foo'), escaped_str('bar')))

    def test_concatenate_from_jbos(self):
        s = jbos('foo', 'bar') + escaped_str('baz')
        self.assertEqual(s.bits, ('foo', 'bar', escaped_str('baz')))

        s = jbos('foo', 'bar') + 'baz'
        self.assertEqual(s.bits, ('foo', 'bar', 'baz'))

        s = escaped_str('foo') + jbos('bar', 'baz')
        self.assertEqual(s.bits, (escaped_str('foo'), 'bar', 'baz'))

        s = 'foo' + jbos('bar', 'baz')
        self.assertEqual(s.bits, ('foo', 'bar', 'baz'))

    def test_safe_str(self):
        self.assertEqual(safe_str('foo'), 'foo')
        self.assertEqual(safe_str(escaped_str('foo')), escaped_str('foo'))
        self.assertEqual(safe_str(jbos('foo')).bits, jbos('foo').bits)
        self.assertRaises(NotImplementedError, safe_str, 123)

    def test_join(self):
        s = join([], ',')
        self.assertEqual(s.bits, ())

        s = join(['foo'], ',')
        self.assertEqual(s.bits, ('foo',))

        s = join(['foo', 'bar'], ',')
        self.assertEqual(s.bits, ('foo', ',', 'bar'))

        s = join([escaped_str('foo'), 'bar'], ',')
        self.assertEqual(s.bits, (escaped_str('foo'), ',', 'bar'))



from itertools import chain
from six.moves import filter as ifilter

from .hooks import builtin
from .. import path
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..build_inputs import build_input
from ..file_types import Directory


@build_input('install')
class InstallTargets(object):
    def __init__(self):
        self.files = []
        self.directories = []

    def __nonzero__(self):
        return bool(self.files or self.directories)


@builtin.globals('builtins', 'build_inputs')
def install(builtins, build, *args, **kwargs):
    def _flatten(args):
        for i in args:
            for j in i.all:
                yield j

    if len(args) == 0:
        raise ValueError('expected at least one argument')
    all_files = kwargs.pop('all', True)

    for i in _flatten(args) if all_files else args:
        if isinstance(i, Directory):
            build['install'].directories.append(i)
        else:
            builtins['default'](i)
            build['install'].files.append(i)


def _install_commands(backend, build_inputs, buildfile, env):
    install_targets = build_inputs['install']
    if not install_targets:
        return None

    install = env.tool('install')
    mkdir_p = env.tool('mkdir_p')

    def install_line(file):
        kind = file.install_kind.upper()
        cmd = backend.cmd_var(install, buildfile)

        if kind != 'PROGRAM':
            kind = 'DATA'
            cmd = [cmd] + install.data_args
        cmd = buildfile.variable('INSTALL_' + kind, cmd,
                                 backend.Section.command, True)

        src = file.path
        dst = path.install_path(file.path, file.install_root)
        return install(cmd, src, dst)

    def mkdir_line(dir):
        src = dir.path
        dst = path.install_path(dir.path.parent(), dir.install_root)
        return mkdir_p.copy(backend.cmd_var(mkdir_p, buildfile), src, dst)

    def post_install(file):
        if file.post_install:
            cmd = backend.cmd_var(file.post_install, buildfile)
            return file.post_install(cmd, file)

    return list(chain(
        (install_line(i) for i in install_targets.files),
        (mkdir_line(i) for i in install_targets.directories),
        ifilter(None, (post_install(i) for i in install_targets.files))
    ))


@make.post_rule
def make_install_rule(build_inputs, buildfile, env):
    recipe = _install_commands(make, build_inputs, buildfile, env)
    if recipe:
        buildfile.rule(
            target='install',
            deps='all',
            recipe=recipe,
            phony=True
        )


@ninja.post_rule
def ninja_install_rule(build_inputs, buildfile, env):
    commands = _install_commands(ninja, build_inputs, buildfile, env)
    if commands:
        ninja.command_build(
            buildfile, env,
            output='install',
            inputs=['all'],
            commands=commands
        )



import os
import unittest

from bfg9000.path import *
from bfg9000.platforms import platform_name

path_variables = {
    Root.srcdir: '$(srcdir)',
    Root.builddir: None,
    InstallRoot.prefix: '$(prefix)',
}


class TestPath(unittest.TestCase):
    def test_realize_srcdir(self):
        p = Path('foo', Root.srcdir)
        self.assertEqual(p.realize(path_variables),
                         os.path.join('$(srcdir)', 'foo'))
        self.assertEqual(p.realize(path_variables, executable=True),
                         os.path.join('$(srcdir)', 'foo'))

        p = Path('foo/bar', Root.srcdir)
        self.assertEqual(p.realize(path_variables),
                         os.path.join('$(srcdir)', 'foo', 'bar'))
        self.assertEqual(p.realize(path_variables, executable=True),
                         os.path.join('$(srcdir)', 'foo', 'bar'))

    def test_realize_builddir(self):
        p = Path('foo', Root.builddir)
        self.assertEqual(p.realize(path_variables), 'foo')
        self.assertEqual(p.realize(path_variables, executable=True),
                         os.path.join('.', 'foo'))

        p = Path('foo/bar', Root.builddir)
        self.assertEqual(p.realize(path_variables), os.path.join('foo', 'bar'))
        self.assertEqual(p.realize(path_variables, executable=True),
                         os.path.join('foo', 'bar'))

    def test_realize_absolute(self):
        p = Path('/foo/bar', Root.builddir)
        self.assertEqual(p.realize(path_variables),
                         os.path.join(os.path.sep, 'foo', 'bar'))
        self.assertEqual(p.realize(path_variables, executable=True),
                         os.path.join(os.path.sep, 'foo', 'bar'))

        if platform_name() == 'windows':
            p = Path(r'C:\foo\bar', Root.builddir)
            self.assertEqual(p.realize(path_variables),
                             os.path.join('C:', os.path.sep, 'foo', 'bar'))
            self.assertEqual(p.realize(path_variables, executable=True),
                             os.path.join('C:', os.path.sep, 'foo', 'bar'))

    def test_realize_srcdir_empty(self):
        p = Path('', Root.srcdir)
        self.assertEqual(p.realize(path_variables), '$(srcdir)')
        self.assertEqual(p.realize(path_variables, executable=True),
                         '$(srcdir)')

    def test_realize_builddir_empty(self):
        p = Path('', Root.builddir)
        self.assertEqual(p.realize(path_variables), '.')
        self.assertEqual(p.realize(path_variables, executable=True), '.')

    def test_parent(self):
        p = Path('foo/bar', Root.srcdir)
        self.assertEqual(p.parent(), Path('foo', Root.srcdir))

        p = Path('bar', Root.srcdir)
        self.assertEqual(p.parent(), Path('', Root.srcdir))

        p = Path('', Root.srcdir)
        self.assertRaises(ValueError, p.parent)

    def test_append(self):
        p = Path('foo', Root.srcdir)
        self.assertEqual(p.append('bar'), Path('foo/bar', Root.srcdir))

    def test_addext(self):
        p = Path('foo', Root.srcdir)
        self.assertEqual(p.addext('.txt'), Path('foo.txt', Root.srcdir))

    def test_basename(self):
        p = Path('foo/bar', Root.srcdir)
        self.assertEqual(p.basename(), 'bar')

    def test_install_path_srcdir(self):
        p = Path('foo/bar', Root.srcdir)
        self.assertEqual(install_path(p, InstallRoot.bindir),
                         Path('bar', InstallRoot.bindir))

    def test_install_path_builddir(self):
        p = Path('foo/bar', Root.builddir)
        self.assertEqual(install_path(p, InstallRoot.bindir),
                         Path('foo/bar', InstallRoot.bindir))



import unittest

from bfg9000.shell.posix import *


class TestSplit(unittest.TestCase):
    def test_single(self):
        self.assertEqual(split('foo'), ['foo'])

    def test_multiple(self):
        self.assertEqual(split('foo bar baz'), ['foo', 'bar', 'baz'])

    def test_quote(self):
        self.assertEqual(split('foo "bar baz"'), ['foo', 'bar baz'])
        self.assertEqual(split('foo"bar baz"'), ['foobar baz'])


class TestQuote(unittest.TestCase):
    def test_simple(self):
        self.assertEqual(quote('foo'), 'foo')

    def test_space(self):
        self.assertEqual(quote('foo bar'), "'foo bar'")

    def test_quote(self):
        self.assertEqual(quote('"foo"'), '\'"foo"\'')

    def test_shell_chars(self):
        self.assertEqual(quote('&&'), "'&&'")
        self.assertEqual(quote('>'), "'>'")
        self.assertEqual(quote('|'), "'|'")



import os.path
import re
from six import assertRegex

from .integration import *


class TestEnvVars(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'env_vars', *args, **kwargs)

    @skip_if_backend('msbuild')
    def test_test(self):
        self.build('test')

    @skip_if_backend('msbuild')
    def test_command(self):
        assertRegex(self,
            self.build('script'),
            re.compile('^hello script$', re.MULTILINE)
        )
        self.assertExists(os.path.join(self.builddir, 'file'))



from .. import safe_str
from .hooks import tool
from ..file_types import SharedLibrary
from ..iterutils import uniques
from ..path import install_path


@tool('patchelf')
class PatchElf(object):
    rule_name = command_var = 'patchelf'

    def __init__(self, env):
        self.command = env.getvar('PATCHELF', 'patchelf')

    def __call__(self, cmd, file):
        paths = uniques(
            install_path(i.path, i.install_root).parent()
            for i in file.creator.all_libs if isinstance(i, SharedLibrary)
        )
        if paths:
            return [cmd, '--set-rpath', safe_str.join(paths, ':'),
                    install_path(file.path, file.install_root)]



from six import iteritems

from .file_types import File, Node, sourcify
from .iterutils import iterate


_build_inputs = {}


def build_input(name):
    def wrapper(fn):
        if name in _build_inputs:
            raise ValueError('"{}" already registered'.format(name))
        _build_inputs[name] = fn
        return fn
    return wrapper


class Edge(object):
    def __init__(self, build, target, extra_deps=None):
        for t in target.all:
            t.creator = self
        self.target = target

        self.extra_deps = [sourcify(i, Node, File)
                           for i in iterate(extra_deps)]
        build.add_edge(self)


class BuildInputs(object):
    def __init__(self):
        self.edges = []
        self.extra_inputs = {}

        for name, fn in iteritems(_build_inputs):
            self.extra_inputs[name] = fn()

    def add_edge(self, edge):
        self.edges.append(edge)

    def __getitem__(self, key):
        return self.extra_inputs[key]



import os.path
from itertools import chain

from .hooks import builtin
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..build_inputs import build_input, Edge
from ..file_types import *
from ..iterutils import iterate, listify, uniques
from ..path import Root
from ..shell import posix as pshell


build_input('link_options')(list)


class Link(Edge):
    __prefixes = {
        'executable': '',
        'static_library': 'lib',
        'shared_library': 'lib',
    }

    @classmethod
    def __name(cls, name, mode):
        head, tail = os.path.split(name)
        return os.path.join(head, cls.__prefixes[mode] + tail)

    def __init__(self, builtins, build, env, mode, name, files, include=None,
                 libs=None, packages=None, compile_options=None,
                 link_options=None, lang=None, extra_deps=None):
        self.name = self.__name(name, mode)
        self.packages = listify(packages)

        # XXX: Try to detect if a string refers to a shared lib?
        self.libs = [sourcify(i, Library, StaticLibrary)
                     for i in iterate(libs)]
        self.all_libs = sum((i.libraries for i in self.packages), self.libs)

        self.files = builtins['object_files'](
            files, include=include, packages=packages, options=compile_options,
            lang=lang
        )
        if ( len(self.files) == 0 and
             not any(isinstance(i, WholeArchive) for i in self.libs) ):
            raise ValueError('need at least one source file')

        langs = chain([lang], (i.lang for i in self.files),
                      (i.lang for i in self.all_libs))
        self.builder = builder = env.linker(langs, mode)

        self.user_options = pshell.listify(link_options)
        self._internal_options = []

        target = self.builder.output_file(name)

        if mode == 'static_library':
            if self.options:
                raise ValueError('link options are not allowed for static ' +
                                 'libraries')
            if self.all_libs:
                raise ValueError('libraries cannot be linked into static ' +
                                 'libraries')
        else:
            lib_dirs = sum((i.lib_dirs for i in self.packages), self.all_libs)
            self._internal_options.extend(builder.lib_dirs(lib_dirs, target))
            self._internal_options.extend(builder.import_lib(target))

            links = sum((builder.link_lib(i) for i in self.all_libs), [])
            self.lib_options = links

        for c in (i.creator for i in self.files if i.creator):
            c.link_options.extend(c.builder.link_args(self.name, mode))

        if getattr(self.builder, 'post_install', None):
            target.post_install = self.builder.post_install

        build['defaults'].add(target)
        Edge.__init__(self, build, target, extra_deps)

    @property
    def options(self):
        return self._internal_options + self.user_options


@builtin.globals('builtins', 'build_inputs', 'env')
def executable(builtins, build, env, name, files=None, **kwargs):
    if files is None and kwargs.get('libs') is None:
        return Executable(name, root=Root.srcdir, **kwargs)
    else:
        return Link(builtins, build, env, 'executable', name, files,
                    **kwargs).target


@builtin.globals('builtins', 'build_inputs', 'env')
def static_library(builtins, build, env, name, files=None, **kwargs):
    if files is None and kwargs.get('libs') is None:
        return StaticLibrary(name, root=Root.srcdir, **kwargs)
    else:
        return Link(builtins, build, env, 'static_library', name, files,
                    **kwargs).target


@builtin.globals('builtins', 'build_inputs', 'env')
def shared_library(builtins, build, env, name, files=None, **kwargs):
    if files is None and kwargs.get('libs') is None:
        # XXX: What to do here for Windows, which has a separate DLL file?
        return SharedLibrary(name, root=Root.srcdir, **kwargs)
    else:
        return Link(builtins, build, env, 'shared_library', name, files,
                    **kwargs).target


@builtin
def whole_archive(lib):
    lib = sourcify(lib, StaticLibrary)
    return WholeArchive(lib)


@builtin.globals('build_inputs')
def global_link_options(build, options):
    build['link_options'].extend(pshell.listify(options))


def _get_flags(backend, rule, build_inputs, buildfile):
    global_ldflags, ldflags = backend.flags_vars(
        rule.builder.link_var + 'flags',
        rule.builder.global_args + build_inputs['link_options'],
        buildfile
    )

    variables = {}
    cmd_kwargs = {'args': ldflags}

    ldflags_value = rule.builder.mode_args + rule.options
    if ldflags_value:
        variables[ldflags] = [global_ldflags] + ldflags_value

    if rule.builder.mode != 'static_library':
        global_ldlibs, ldlibs = backend.flags_vars(
            rule.builder.link_var + 'libs', rule.builder.global_libs, buildfile
        )
        cmd_kwargs['libs'] = ldlibs
        if rule.lib_options:
            variables[ldlibs] = [global_ldlibs] + rule.lib_options

    return variables, cmd_kwargs


@make.rule_handler(Link)
def make_link(rule, build_inputs, buildfile, env):
    linker = rule.builder
    variables, cmd_kwargs = _get_flags(make, rule, build_inputs, buildfile)

    recipename = make.var('RULE_{}'.format(linker.rule_name.upper()))
    if not buildfile.has_variable(recipename):
        buildfile.define(recipename, [
            linker(cmd=make.cmd_var(linker, buildfile), input=make.var('1'),
                   output=make.var('2'), **cmd_kwargs)
        ])

    recipe = make.Call(recipename, rule.files, rule.target.path)
    if len(rule.target.all) > 1:
        target = rule.target.path.addext('.stamp')
        buildfile.rule(target=rule.target.all, deps=[target])
        recipe = [recipe, make.silent([ 'touch', make.var('@') ])]
    else:
        target = rule.target

    dirs = uniques(i.path.parent() for i in rule.target.all)
    buildfile.rule(
        target=target,
        deps=rule.files + rule.libs + rule.extra_deps,
        order_only=[i.append(make.dir_sentinel) for i in dirs if i],
        recipe=recipe,
        variables=variables
    )


@ninja.rule_handler(Link)
def ninja_link(rule, build_inputs, buildfile, env):
    linker = rule.builder
    variables, cmd_kwargs = _get_flags(ninja, rule, build_inputs, buildfile)
    variables[ninja.var('output')] = rule.target.path

    if not buildfile.has_rule(linker.rule_name):
        buildfile.rule(name=linker.rule_name, command=linker(
            cmd=ninja.cmd_var(linker, buildfile), input=ninja.var('in'),
            output=ninja.var('output'), **cmd_kwargs
        ))

    buildfile.build(
        output=rule.target.all,
        rule=linker.rule_name,
        inputs=rule.files,
        implicit=rule.libs + rule.extra_deps,
        variables=variables
    )

try:
    from ..backends.msbuild import writer as msbuild

    def _reduce_options(files, global_options):
        compilers = uniques(i.creator.builder for i in files)
        langs = uniques(i.lang for i in files)

        per_file_opts = []
        for i in files:
            # We intentionally exclude internal_options, since MSBuild handles
            # these its own way.
            for opts in [i.creator.link_options, i.creator.user_options]:
                if opts not in per_file_opts:
                    per_file_opts.append(opts)

        return list(chain(
            chain.from_iterable(i.global_args for i in compilers),
            chain.from_iterable(global_options[i] for i in langs),
            chain.from_iterable(per_file_opts)
        ))

    @msbuild.rule_handler(Link)
    def msbuild_link(rule, build_inputs, solution, env):
        # By definition, a dependency for an edge must already be defined by
        # the time the edge is created, so we can map *all* the dependencies to
        # their associated projects by looking at the projects we've already
        # created.
        dependencies = []
        for dep in rule.libs:
            if dep.creator.target not in solution:
                raise ValueError('unknown dependency for {!r}'.format(dep))
            dependencies.append(solution[dep.creator.target])

        mode = {
            'executable'    : 'Application',
            'static_library': 'StaticLibrary',
            'shared_library': 'DynamicLibrary',
        }[rule.builder.mode]

        includes = uniques(chain.from_iterable(
            i.creator.all_includes for i in rule.files
        ))

        project = msbuild.VcxProject(
            name=rule.name,
            version=env.getvar('VISUALSTUDIOVERSION'),
            mode=mode,
            platform=env.getvar('PLATFORM'),
            output_file=rule.target,
            srcdir=env.srcdir.string(),
            files=[i.creator.file for i in rule.files],
            compile_options=_reduce_options(
                rule.files, build_inputs['compile_options']
            ),
            includes=includes,
            # We intentionally exclude internal_options from the link step,
            # since MSBuild handles these its own way.
            link_options=(
                rule.user_options + rule.builder.global_args +
                build_inputs['link_options']
            ),
            libs=rule.all_libs,
            lib_dirs=sum((i.lib_dirs for i in rule.packages), []),
            dependencies=dependencies,
        )
        solution[rule.target] = project
except:
    pass



import argparse
import os
import re
import sys

from . import builtins
from .backends import get_backends
from .build_inputs import BuildInputs
from .environment import Environment, EnvVersionError
from .path import Path, InstallRoot
from .platforms import platform_info
from .version import version

bfgfile = 'build.bfg'


def is_srcdir(path):
    return os.path.exists(os.path.join(path, bfgfile))


def samefile(path1, path2):
    if hasattr(os.path, 'samefile'):
        return os.path.samefile(path1, path2)
    else:
        # This isn't entirely accurate, but it's close enough, and should only
        # be necessary for Windows with Python 2.x.
        return os.path.realpath(path1) == os.path.realpath(path2)


def parse_args(parser, args=None, namespace=None):
    def check_dir(path, check_exist=False):
        if not os.path.exists(path):
            parser.error("'{}' does not exist".format(path))
        if not os.path.isdir(path):
            parser.error("'{}' is not a directory".format(path))

    args = parser.parse_args(args, namespace)

    if not args.regenerate:
        if not args.srcdir:
            parser.error('at least one of srcdir or builddir must be defined')

        if args.builddir:
            check_dir(args.srcdir, check_exist=True)
        else:
            args.builddir = '.'
            if not is_srcdir(args.srcdir):
                args.srcdir, args.builddir = args.builddir, args.srcdir

        if os.path.exists(args.builddir):
            check_dir(args.builddir)
            if samefile(args.srcdir, args.builddir):
                parser.error('source and build directories must be different')

        if not is_srcdir(args.srcdir):
            parser.error('source directory must contain a build.bfg file')
        if is_srcdir(args.builddir):
            parser.error('build directory must not contain a build.bfg file')

        if not os.path.exists(args.builddir):
            os.mkdir(args.builddir)
        args.srcdir = Path(os.path.abspath(args.srcdir))
        args.builddir = Path(os.path.abspath(args.builddir))
    else:
        args.srcdir, args.builddir = None, args.srcdir
        if args.srcdir:
            parser.error('source directory cannot be passed when regenerating')
        if not args.builddir:
            args.builddir = '.'

        check_dir(args.builddir, check_exist=True)
        args.builddir = Path(os.path.abspath(args.builddir))

    return args


def execute_script(env, filename=bfgfile):
    build = BuildInputs()
    builtin_dict = builtins.bind(build_inputs=build, env=env)

    bfgpath = env.srcdir.append(filename).string()
    with open(bfgpath, 'r') as f:
        os.chdir(env.srcdir.string())
        code = compile(f.read(), filename, 'exec')
        try:
            exec(code, builtin_dict, {})
        except SystemExit:
            pass

    return build


def main():
    backends = get_backends()
    install_dirs = platform_info().install_dirs
    path_help = 'installation path for {} (default: %(default)r)'

    def path_arg(value):
        return Path(os.path.abspath(value))

    parser = argparse.ArgumentParser(prog='bfg9000')
    parser.add_argument('srcdir', nargs='?', help='source directory')
    parser.add_argument('builddir', nargs='?', help='build directory')
    parser.add_argument('--version', action='version',
                        version='%(prog)s ' + version)
    parser.add_argument('--backend', choices=list(backends.keys()),
                        default=list(backends.keys())[0],
                        help='backend (default: %(default)s)')
    parser.add_argument('--prefix', type=path_arg, metavar='PATH',
                        default=install_dirs[InstallRoot.prefix],
                        help='installation prefix (default: %(default)r)')
    parser.add_argument('--bindir', type=path_arg, metavar='PATH',
                        default=install_dirs[InstallRoot.bindir],
                        help=path_help.format('executables'))
    parser.add_argument('--libdir', type=path_arg, metavar='PATH',
                        default=install_dirs[InstallRoot.libdir],
                        help=path_help.format('libraries'))
    parser.add_argument('--includedir', type=path_arg, metavar='PATH',
                        default=install_dirs[InstallRoot.includedir],
                        help=path_help.format('headers'))
    parser.add_argument('--regenerate', action='store_true',
                        help='regenerate build files')

    args = parse_args(parser)
    if args.regenerate:
        try:
            env = Environment.load(args.builddir.string())
        except Exception as e:
            sys.stderr.write('{prog}: error loading environment: {msg}\n'
                             .format(prog=parser.prog, msg=e))
            if isinstance(e, EnvVersionError):
                sys.stderr.write('Please re-run bfg9000 manually.\n')
            return 1
    else:
        # De-munge the entry point if we're on Windows.
        bfgpath = Path(os.path.realpath(
            re.sub('-script.py$', '.exe', sys.argv[0])
        ))
        env = Environment(
            bfgpath=bfgpath,
            backend=args.backend,
            backend_version=backends[args.backend].version(),
            srcdir=args.srcdir,
            builddir=args.builddir,
            install_dirs={
                InstallRoot.prefix: args.prefix,
                InstallRoot.bindir: args.bindir,
                InstallRoot.libdir: args.libdir,
                InstallRoot.includedir: args.includedir,
            }
        )
        env.save(args.builddir.string())

    build = execute_script(env)
    backends[env.backend].write(env, build)



import os
import re
import subprocess
from packaging.version import Version

from .syntax import *
from ...pathutils import makedirs
from ...platforms import which


def version(env=os.environ):
    try:
        msbuild = which(env.get('MSBUILD', ['msbuild', 'xbuild']), env)
        output = subprocess.check_output(
            [msbuild, '/version'],
            universal_newlines=True
        )
        m = re.search(r'([\d\.]+)$', output)
        if m:
            return Version(m.group(1))
    except IOError:
        return None

priority = 1

_rule_handlers = {}


def rule_handler(rule_name):
    def decorator(fn):
        _rule_handlers[rule_name] = fn
        return fn
    return decorator


def write(env, build_inputs):
    uuids = UuidMap(env.builddir.append('.bfg_uuid').string())
    solution = Solution(uuids)

    for e in build_inputs.edges:
        if type(e) in _rule_handlers:
            _rule_handlers[type(e)](e, build_inputs, solution, env)

    # XXX: Handle default builds. Default builds go first in the solution. This
    # also means we'd need to support aliases so that we can have multiple
    # builds be the default.
    with open(env.builddir.append('project.sln').string(), 'w') as out:
        solution.write(out)
    for p in solution:
        projfile = env.builddir.append(p.path).string()
        makedirs(os.path.dirname(projfile), exist_ok=True)
        with open(projfile, 'w') as out:
            p.write(out)
    uuids.save()



import os
import shutil
import subprocess
import time
import unittest
from collections import namedtuple

from six import iteritems

from bfg9000.backends import get_backends
from bfg9000.environment import Environment
from bfg9000.path import InstallRoot
from bfg9000.pathutils import makedirs
from bfg9000.platforms import platform_info, platform_name

this_dir = os.path.abspath(os.path.dirname(__file__))
examples_dir = os.path.join(this_dir, '..', '..', 'examples')
test_data_dir = os.path.join(this_dir, '..', 'data')
test_stage_dir = os.path.join(this_dir, '..', 'stage')

env = Environment(None, None, None, None, None, None)

Target = namedtuple('Target', ['name', 'path'])

if os.getenv('BACKENDS', '').strip():
    backends = os.getenv('BACKENDS').split(' ')
else:
    backends = [k for k, v in iteritems(get_backends()) if v.priority > 0]


def cleandir(path, recreate=True):
    if os.path.exists(path):
        # Windows seems to keep an executable file open a little bit after the
        # process returns from wait(), so try a few times, sleeping a bit in
        # between.
        for t in [0.1, 0.25, 0.5, 1.0, 2.0, None]:
            try:
                shutil.rmtree(path)
                break
            except OSError:
                if t is None:
                    raise RuntimeError('unable to remove {}'.format(path))
                time.sleep(t)
    if recreate:
        makedirs(path)


def skip_pred(predicate, msg='skipped'):
    def wrap(fn):
        def inner(self, *args, **kwargs):
            if predicate(self):
                raise unittest.SkipTest(msg)
            return fn(self, *args, **kwargs)
        return inner
    return wrap


def skip_if_backend(backend):
    return skip_pred(lambda x: x.backend == backend,
                     'not supported for backend "{}"'.format(backend))


def only_if_backend(backend):
    return skip_pred(lambda x: x.backend != backend,
                     'only supported for backend "{}"'.format(backend))


def xfail_if(xfail):
    def wrap(fn):
        if xfail:
            return unittest.expectedFailure(fn)
        else:
            return fn
    return wrap


def xfail_if_platform(platform):
    return xfail_if(platform_name() == platform)


class SubprocessError(unittest.TestCase.failureException):
    def __init__(self, message):
        unittest.TestCase.failureException.__init__(
            self, '\n{line}\n{msg}\n{line}'.format(line='-' * 60, msg=message)
        )


class IntegrationTest(unittest.TestCase):
    def __init__(self, srcdir, *args, **kwargs):
        dist = kwargs.pop('dist', False)
        self._args = args
        self._kwargs = kwargs

        self.stage_src = kwargs.pop('stage_src', False)
        self.backend = kwargs.pop('backend', None)
        unittest.TestCase.__init__(self, *args, **kwargs)
        if self.backend is None:
            return

        self.extra_args = []

        srcname = os.path.basename(srcdir)
        self.srcdir = os.path.join(test_data_dir, srcdir)
        if self.stage_src:
            self.orig_srcdir = self.srcdir
            self.srcdir = os.path.join(test_stage_dir, srcname)

        self.builddir = os.path.join(test_stage_dir, srcname + '-build')

        if dist:
            self.distdir = os.path.join(test_stage_dir, srcname + '-dist')
            self.extra_args = ['--prefix', self.distdir]

            install_dirs = platform_info().install_dirs
            path_vars = {InstallRoot.prefix: self.distdir}
            for i in InstallRoot:
                setattr(self, i.name, install_dirs[i].realize(path_vars))

    def parameterize(self):
        return [ self.__class__(backend=i, *self._args, **self._kwargs)
                 for i in backends ]

    def shortDescription(self):
        return self.backend

    def _target_name(self, target):
        if self.backend == 'msbuild':
            if isinstance(target, Target):
                target = target.name
            return '/target:' + target
        else:
            if isinstance(target, Target):
                target = target.path
            return target

    def _target_path(self, target):
        if isinstance(target, Target):
            prefix = '.'
            if self.backend == 'msbuild':
                prefix = 'Debug'
                if os.getenv('PLATFORM') == 'X64':
                    prefix = os.path.join('X64', prefix)
            return os.path.join(prefix, target.path)
        return target

    def setUp(self):
        if self.stage_src:
            cleandir(self.srcdir, recreate=False)
            shutil.copytree(self.orig_srcdir, self.srcdir)
        os.chdir(self.srcdir)
        cleandir(self.builddir)
        self.assertPopen(
            ['bfg9000', self.srcdir, self.builddir, '--backend',
             self.backend] + self.extra_args
        )
        os.chdir(self.builddir)

    def build(self, target=None):
        args = [os.getenv(self.backend.upper(), self.backend)]
        if target:
            args.append(self._target_name(target))
        return self.assertPopen(args, True)

    def wait(self, t=1):
        time.sleep(t)

    def assertPopen(self, args, bad=False):
        args = [self._target_path(i) for i in args]
        proc = subprocess.Popen(
            args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
            universal_newlines=True
        )
        output = proc.communicate()[0]
        if proc.returncode != 0:
            raise SubprocessError(output)
        return output

    def assertOutput(self, args, output):
        args = [self._target_path(i) for i in args]
        self.assertEqual(subprocess.check_output(
            args, stderr=subprocess.STDOUT, universal_newlines=True
        ), output)

    def assertExists(self, path):
        if not os.path.exists(self._target_path(path)):
            raise unittest.TestCase.failureException(
                "'{}' does not exist".format(os.path.normpath(path))
            )

    def assertNotExists(self, path):
        if os.path.exists(self._target_path(path)):
            raise unittest.TestCase.failureException(
                "'{}' exists".format(os.path.normpath(path))
            )


def executable(name):
    return Target(name, os.path.normpath(os.path.join(
        '.', name + platform_info().executable_ext
    )))


if platform_info().name == 'windows' and env.compiler('c++').flavor == 'msvc':
    _library_prefix = ''
else:
    _library_prefix = 'lib'


def shared_library(name):
    head, tail = os.path.split(name)
    return Target(name, os.path.normpath(os.path.join(
        '.', head, _library_prefix + tail + platform_info().shared_library_ext
    )))


def static_library(name):
    suffix = '.lib' if platform_info().name == 'windows' else '.a'
    head, tail = os.path.split(name)
    return Target(name, os.path.normpath(os.path.join(
        '.', head, _library_prefix + tail + suffix
    )))


def load_tests(loader, standard_tests, pattern):
    all_tests = unittest.TestSuite()
    for suite in standard_tests:
        for case in suite:
            all_tests.addTests(case.parameterize())
    return all_tests



from .hooks import tool


@tool('mkdir_p')
class MkdirP(object):
    rule_name = command_var = 'mkdir_p'

    def __init__(self, env):
        if env.platform.name == 'windows':
            default = env.bfgpath.parent().append('bfg9000-makedirs')
        else:
            default = 'mkdir -p'
        self.command = env.getvar('MKDIR_P', default)

    def __call__(self, cmd, path):
        return [cmd, path]

    # XXX: Remove this once we rewrite the install command.
    def copy(self, cmd, src, dst):
        return cmd + ' ' + dst + ' && cp -r ' + src + '/* ' + dst



import os.path
import re
from six import assertRegex

from .integration import *


class TestCommand(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(
            self, os.path.join(examples_dir, '07_commands'), *args, **kwargs
        )

    @skip_if_backend('msbuild')
    def test_hello(self):
        assertRegex(self,
            self.build('hello'),
            re.compile('^hello$', re.MULTILINE)
        )

    @skip_if_backend('msbuild')
    def test_world(self):
        assertRegex(self,
            self.build('world'),
            re.compile('^world$', re.MULTILINE)
        )

    @skip_if_backend('msbuild')
    def test_script(self):
        assertRegex(self,
            self.build('script'),
            re.compile('^hello, world!$', re.MULTILINE)
        )
        self.assertExists(os.path.join(self.builddir, 'file'))

    @skip_if_backend('msbuild')
    def test_alias(self):
        output = self.build('hello-world')
        assertRegex(self, output, re.compile('^hello$', re.MULTILINE))
        assertRegex(self, output, re.compile('^world$', re.MULTILINE))



import os.path
import re
import subprocess
from itertools import chain
from six.moves import filter as ifilter

from .utils import library_macro
from ..file_types import *
from ..iterutils import iterate, listify, uniques
from ..path import Root
from ..platforms import platform_name


class CcCompiler(object):
    def __init__(self, env, lang, name, command, cflags):
        self.platform = env.platform
        self.lang = lang

        self.rule_name = self.command_var = name
        self.command = command

        self.global_args = cflags

    @property
    def flavor(self):
        return 'cc'

    @property
    def deps_flavor(self):
        return 'gcc'

    def __call__(self, cmd, input, output, deps=None, args=None):
        result = [cmd]
        result.extend(iterate(args))
        result.extend(['-c', input])
        if deps:
            result.extend(['-MMD', '-MF', deps])
        result.extend(['-o', output])
        return result

    def output_file(self, name):
        return ObjectFile(name + '.o', Root.builddir, self.lang)

    def include_dir(self, directory):
        if directory.system:
            return ['-isystem' + directory.path]
        else:
            return ['-I' + directory.path]

    def link_args(self, name, mode):
        if mode == 'executable':
            return []
        elif mode in ['shared_library', 'static_library']:
            args = ['-fPIC']
            if self.platform.has_import_library:
                args.append('-D' + library_macro(name, mode))
            return args
        else:
            raise ValueError("unknown mode '{}'".format(mode))


class CcLinker(object):
    def __init__(self, env, mode, lang, name, command, ldflags, ldlibs):
        self.env = env
        self.mode = mode
        self.lang = lang

        self.rule_name = 'link_' + name
        self.command_var = name
        self.command = command
        self.link_var = 'ld'

        self.global_args = ldflags
        self.global_libs = ldlibs

        # Create a regular expression to extract the library name for linking
        # with -l.
        lib_formats = [r'lib(.*)\.a']
        if not self.platform.has_import_library:
            so_ext = re.escape(self.platform.shared_library_ext)
            lib_formats.append(r'lib(.*)' + so_ext)
        # XXX: Include Cygwin here too?
        if self.platform.name == 'windows':
            lib_formats.append(r'(.*)\.lib')
        self._lib_re = re.compile('(?:' + '|'.join(lib_formats) + ')$')

    def _extract_lib_name(self, library):
        basename = library.link.path.basename()
        m = self._lib_re.match(basename)
        if not m:
            raise ValueError("'{}' is not a valid library name"
                             .format(basename))

        # Get the first non-None group from the match.
        return next(ifilter( None, m.groups() ))

    @property
    def platform(self):
        return self.env.platform

    @property
    def flavor(self):
        return 'cc'

    def __call__(self, cmd, input, output, libs=None, args=None):
        result = [cmd]
        result.extend(iterate(args))
        result.extend(iterate(input))
        result.extend(iterate(libs))
        result.extend(['-o', output])
        return result

    @property
    def post_install(self):
        if self.platform.has_rpath:
            return self.env.tool('patchelf')
        return None

    @property
    def auto_link(self):
        return False

    def output_file(self, name):
        if self.mode == 'executable':
            return Executable(
                name + self.platform.executable_ext, Root.builddir, self.lang
            )
        elif self.mode == 'shared_library':
            head, tail = os.path.split(name)

            def lib(prefix='lib'):
                return os.path.join(
                    head, prefix + tail + self.platform.shared_library_ext
                )

            if self.platform.has_import_library:
                dllprefix = 'cyg' if self.platform.name == 'cygwin' else 'lib'
                return DllLibrary(lib(dllprefix), lib() + '.a', Root.builddir,
                                  self.lang)
            else:
                return SharedLibrary(lib(), Root.builddir, self.lang)
        else:
            raise ValueError("unknown mode '{}'".format(self.mode))

    @property
    def mode_args(self):
        return ['-shared', '-fPIC'] if self.mode == 'shared_library' else []

    def lib_dirs(self, libraries, target):
        def get_dir(lib):
            return lib.path.parent() if isinstance(lib, Library) else lib

        libraries = listify(libraries)
        dirs = uniques(get_dir(i) for i in iterate(libraries)
                       if not isinstance(i, StaticLibrary))
        result = ['-L' + i for i in dirs]

        if self.platform.has_rpath:
            start = target.path.parent()
            paths = uniques(i.path.parent().relpath(start) for i in libraries
                            if isinstance(i, SharedLibrary))
            if paths:
                base = '$ORIGIN'
                result.append('-Wl,-rpath={}'.format( ':'.join(
                    base if i == '.' else os.path.join(base, i) for i in paths
                ) ))

        return result

    def link_lib(self, library):
        if isinstance(library, WholeArchive):
            if platform_name() == 'darwin':
                return ['-Wl,-force_load', library.link.path]
            return ['-Wl,--whole-archive', library.link.path,
                    '-Wl,--no-whole-archive']
        elif isinstance(library, StaticLibrary):
            return [library.link.path]

        # If we're here, we have a SharedLibrary (or possibly just a Library
        # in the case of MinGW).
        return ['-l' + self._extract_lib_name(library)]

    def import_lib(self, library):
        if self.platform.has_import_library and self.mode == 'shared_library':
            return ['-Wl,--out-implib=' + library.import_lib.path]
        return []


class CcPackageResolver(object):
    def __init__(self, env, lang, cmd):
        value = env.getvar('CPATH')
        include_dirs = value.split(os.pathsep) if value else []

        self.include_dirs = [i for i in uniques(chain(
            include_dirs, env.platform.include_dirs
        )) if os.path.exists(i)]

        try:
            # XXX: Will this work for cross-compilation?
            output = subprocess.check_output(
                [cmd, '-print-search-dirs'],
                universal_newlines=True
            )
            m = re.search(r'^libraries: (.*)', output, re.MULTILINE)
            system_lib_dirs = re.split(os.pathsep, m.group(1))
        except:
            system_lib_dirs = []

        value = env.getvar('LIBRARY_PATH')
        user_lib_dirs = value.split(os.pathsep) if value else []

        # XXX: Handle sysroot one day?
        all_lib_dirs = ( os.path.abspath(re.sub('^=', '', i))
                         for i in chain(user_lib_dirs, system_lib_dirs) )
        self.lib_dirs = [i for i in uniques(chain(
            all_lib_dirs, env.platform.lib_dirs
        )) if os.path.exists(i)]

        self.lang = lang
        self.platform = env.platform

    def header(self, name, search_dirs=None):
        if search_dirs is None:
            search_dirs = self.include_dirs

        for base in search_dirs:
            if os.path.exists(os.path.join(base, name)):
                return HeaderDirectory(base, Root.absolute, system=True)

        raise ValueError("unable to find header '{}'".format(name))

    def library(self, name, kind='any', search_dirs=None):
        if search_dirs is None:
            search_dirs = self.lib_dirs

        libnames = []
        if kind in ('any', 'shared'):
            libname = 'lib' + name + self.platform.shared_library_ext
            if self.platform.has_import_library:
                libnames.append((libname + '.a', ImportLibrary))
            else:
                libnames.append((libname, SharedLibrary))
        if kind in ('any', 'static'):
            libnames.append(('lib' + name + '.a', StaticLibrary))

        # XXX: Include Cygwin here too?
        if self.platform.name == 'windows':
            # We don't actually know what kind of library this is. It could be
            # a static library or an import library (which we classify as a
            # kind of shared lib).
            libnames.append((name + '.lib', Library))

        for base in search_dirs:
            for libname, libkind in libnames:
                fullpath = os.path.join(base, libname)
                if os.path.exists(fullpath):
                    return libkind(fullpath, Root.absolute, self.lang)

        raise ValueError("unable to find library '{}'".format(name))



import argparse
import sys

from enum import Enum

from .version import version

Token = Enum('Token', ['char', 'colon', 'space', 'newline'])
State = Enum('State', ['target', 'between', 'dep', 'end'])

# Munge the depfile so that it works a little better under Make. Specifically,
# we need all the dependencies in the depfile to also be targets, so that we
# don't get an error if a dep is removed. For a more-detailed discussion of why
# this is necessary, see <http://scottmcpeak.com/autodepend/autodepend.html>.


def tokenize(s):
    # The depfile syntax is a bit weird, since it seems no one quite
    # understands the correct ways to escape characters for Make in all cases
    # (made worse by the fact that even GNU Make's behavior varies across
    # versions). For our purposes though, we only need to recognize when
    # unescaped colons (always followed by spaces in the depfile generators)
    # and unescaped spaces are emitted.

    s = iter(s)
    while True:
        c = next(s, None)
        if c is None:
            return

        if c == ':':
            c = next(s)
            if c == ' ':
                yield (Token.colon, None)
                continue
            else:
                yield (Token.char, ':')

        if c == '\\':
            c = next(s)
            if c != '\n':  # Swallow escaped newlines.
                yield (Token.char, '\\')
                yield (Token.char, c)
        elif c in ' \t':
            yield (Token.space, None)
        elif c == '\n':
            yield (Token.newline, None)
        else:
            yield (Token.char, c)


def emit_deps(instream, outstream):
    state = State.target

    for tok, value in tokenize(instream.read()):
        if state == State.target:
            if tok == Token.colon:
                state = State.between
            elif tok != Token.char:
                raise Exception(tok)
        elif state == State.dep:
            if tok == Token.char:
                outstream.write(value)
            elif tok == Token.space:
                outstream.write(':\n')
                state = State.between
            elif tok == Token.newline:
                outstream.write(':\n')
                state = State.end
            else:
                raise Exception(tok)
        elif state == State.between:
            if tok == Token.char:
                state = State.dep
                outstream.write(value)
            elif tok == Token.newline:
                state = State.end
            elif tok != Token.space:
                raise Exception(tok)
        else:  # state == State.end
            if tok != Token.newline:
                raise Exception(tok)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--version', action='version',
                        version='%(prog)s ' + version)
    parser.parse_args()

    emit_deps(sys.stdin, sys.stdout)



import platform
import re
import subprocess
from setuptools import setup, find_packages, Command
from bfg9000.version import version

platform_name = platform.system()


class DocServe(Command):
    description = 'serve the documentation locally'
    user_options = [
        ('dev-addr=', None, 'address to host the documentation on'),
    ]

    def initialize_options(self):
        self.dev_addr = '0.0.0.0:8000'

    def finalize_options(self):
        pass

    def run(self):
        subprocess.call(['mkdocs', 'serve', '--dev-addr=' + self.dev_addr])


class DocDeploy(Command):
    description = 'push the documentation to GitHub'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        subprocess.call(['mkdocs', 'gh-deploy', '--clean'])


custom_cmds = {
    'doc_serve': DocServe,
    'doc_deploy': DocDeploy,
}

try:
    from flake8.main import Flake8Command

    class LintCommand(Flake8Command):
        def distribution_files(self):
            return ['setup.py', 'bfg9000', 'examples', 'test']

    custom_cmds['lint'] = LintCommand
except:
    pass

extra_scripts = []

if platform_name == 'Windows':

    extra_scripts.extend([
        'bfg9000-setenv=bfg9000.setenv:main',
        'bfg9000-makedirs=bfg9000.makedirs:main'
    ])

elif platform_name == 'Linux':

    import hashlib
    import os
    import shutil
    import tarfile
    from setuptools.command.develop import develop as DevelopCommand
    from setuptools.command.install import install as InstallCommand

    try:
        from urllib.request import urlretrieve
    except ImportError:
        from urllib import urlretrieve

    from bfg9000.pathutils import makedirs, pushd

    class InstallPatchelf(Command):
        description = 'install the patchelf executable'
        user_options = [
            ('force', 'f', 'force installation of patchelf, even if already ' +
             'installed'),
            ('prefix=', None, 'prefix to install patchelf to')
        ]

        patchelf_url = ('https://nixos.org/releases/patchelf/patchelf-0.8/' +
                        'patchelf-0.8.tar.gz')
        sha256_hash = ('14af06a2da688d577d64ff8dac065bb8' +
                       '903bbffbe01d30c62df7af9bf4ce72fe')

        def initialize_options(self):
            self.force = False
            self.prefix = os.getenv('VIRTUAL_ENV')

        def finalize_options(self):
            pass

        @staticmethod
        def sha256sum(filename, blocksize=65536):
            sha = hashlib.sha256()
            with open(filename, 'rb') as f:
                for block in iter(lambda: f.read(blocksize), b""):
                    sha.update(block)
            return sha.hexdigest()

        def run(self):
            if not self.force:
                try:
                    output = subprocess.check_output(['which', 'patchelf'])
                    print('Found patchelf at {}'.format(output.strip()))
                    return
                except:
                    print('patchelf not found, installing...')

            filename = os.path.basename(self.patchelf_url)
            makedirs('build', exist_ok=True)
            with pushd('build'):
                print('Downloading {}...'.format(self.patchelf_url))
                urlretrieve(self.patchelf_url, filename)

                if self.sha256sum(filename) != self.sha256_hash:
                    raise RuntimeError(
                        "{} doesn't match checksum".format(filename)
                    )

                tar = tarfile.open(filename, 'r:gz')
                sub = tar.getnames()[0]

                if os.path.exists(sub):
                    print('Cleaning {}'.format(sub))
                    shutil.rmtree(sub)

                print('Extracting to build/{}'.format(filename))
                tar.extractall('.')

                with pushd(tar.getnames()[0]):
                    configure = ['./configure']
                    if self.prefix:
                        configure += ['--prefix', self.prefix]

                    print('Configuring: {}'.format(' '.join(configure)))
                    subprocess.check_call(configure)

                    print('Building...')
                    subprocess.check_call(['make'])

                    print('Installing...')
                    subprocess.check_call(['make', 'install'])

    no_patchelf = os.getenv('NO_PATCHELF') in ['1', 'true']

    class Develop(DevelopCommand):
        def run(self):
            DevelopCommand.run(self)
            if not no_patchelf:
                self.run_command('install_patchelf')

    class Install(InstallCommand):
        def run(self):
            InstallCommand.run(self)
            if not no_patchelf:
                self.run_command('install_patchelf')

    custom_cmds.update({
        'install_patchelf': InstallPatchelf,
        'develop': Develop,
        'install': Install,
    })

with open('README.md', 'r') as f:
    # Read from the file and strip out the badges.
    long_desc = re.sub(r'(^# bfg9000.*)\n\n(.+\n)*', r'\1', f.read())

try:
    import pypandoc
    long_desc = pypandoc.convert(long_desc, 'rst', format='md')
except ImportError:
    pass

setup(
    name='bfg9000',
    version=version,

    description='A cross-platform build file generator',
    long_description=long_desc,
    keywords='build file generator',
    url='http://jimporter.github.io/bfg9000/',

    author='Jim Porter',
    author_email='porterj@alum.rit.edu',
    license='BSD',

    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',

        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: BSD License',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    packages=find_packages(exclude=['test', 'test.*']),

    install_requires=['enum-compat', 'packaging', 'six'],
    extras_require={'msbuild': ['lxml']},

    entry_points={
        'console_scripts': [
            'bfg9000=bfg9000.driver:main',
            'bfg9000-depfixer=bfg9000.depfixer:main',
        ] + extra_scripts,
        'bfg9000.backends': [
            'make=bfg9000.backends.make.writer',
            'ninja=bfg9000.backends.ninja.writer',
            'msbuild=bfg9000.backends.msbuild.writer [msbuild]',
        ],
    },

    test_suite='test',
    cmdclass=custom_cmds,
)



from collections import OrderedDict
from pkg_resources import iter_entry_points, DistributionNotFound


def get_backends():
    backends = []
    for i in iter_entry_points('bfg9000.backends'):
        try:
            backend = i.load()
            backends.append((i.name, backend))
        except DistributionNotFound:
            pass

    def sort_key(x):
        return x[1].priority if x[1].version() else 0
    backends.sort(key=sort_key, reverse=True)
    return OrderedDict(backends)



import os.path

from .integration import *

is_mingw = platform_name() == 'windows' and env.compiler('c++').flavor == 'cc'


class TestSystemLibrary(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(
            self, os.path.join(examples_dir, '04_external_library'),
            *args, **kwargs
        )

    @unittest.skipIf(is_mingw, 'xfail on mingw')
    def test_build(self):
        self.build()
        self.assertOutput([executable('program')], '')


class TestBoostLibrary(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'boost', *args, **kwargs)

    @unittest.skipIf(is_mingw, 'xfail on mingw')
    def test_build(self):
        self.build()
        self.assertOutput([executable('program'), '--hello'],
                          'Hello, world!\n')



import os.path
from enum import Enum
from itertools import chain

from . import safe_str

Root = Enum('Root', ['srcdir', 'builddir', 'absolute'])
InstallRoot = Enum('InstallRoot', ['prefix', 'bindir', 'libdir', 'includedir'])


class Path(safe_str.safe_string):
    def __init__(self, path, root=Root.builddir):
        self.suffix = os.path.normpath(path)
        if self.suffix == '.':
            self.suffix = ''

        if os.path.isabs(path):
            self.root = Root.absolute
        elif root == Root.absolute:
            raise ValueError("'{}' is not absolute".format(path))
        else:
            self.root = root

    def parent(self):
        if not self.suffix:
            raise ValueError('already at root')
        return Path(os.path.dirname(self.suffix), self.root)

    def append(self, path):
        return Path(os.path.join(self.suffix, path), self.root)

    def addext(self, ext):
        return Path(self.suffix + ext, self.root)

    def basename(self):
        return os.path.basename(self.suffix)

    def relpath(self, start):
        if os.path.isabs(self.suffix):
            return self.suffix
        else:
            if self.root != start.root:
                raise ValueError('source mismatch')
            return os.path.relpath(self.suffix or '.', start.suffix or '.')

    def to_json(self):
        return (self.suffix, self.root.name)

    @staticmethod
    def from_json(data):
        try:
            base = Root[data[1]]
        except:
            base = InstallRoot[data[1]]
        return Path(data[0], base)

    def realize(self, variables, executable=False):
        root = variables[self.root] if self.root != Root.absolute else None
        if executable and root is None and os.path.sep not in self.suffix:
            root = '.'

        if root is None:
            return self.suffix or '.'
        if not self.suffix:
            return root
        return root + os.path.sep + self.suffix

    def string(self):
        return self.realize(None)

    def __str__(self):
        raise NotImplementedError()

    def __repr__(self):
        variables = {i: '$({})'.format(i.name) for i in
                     chain(Root, InstallRoot)}
        return '`{}`'.format(self.realize(variables))

    def __hash__(self):
        return hash(self.suffix)

    def __eq__(self, rhs):
        return self.root == rhs.root and self.suffix == rhs.suffix

    def __nonzero__(self):
        return self.__bool__()

    def __bool__(self):
        return self.root != Root.builddir or bool(self.suffix)

    def __add__(self, rhs):
        return safe_str.jbos(self, rhs)

    def __radd__(self, lhs):
        return safe_str.jbos(lhs, self)


def install_path(path, install_root):
    if path.root == Root.srcdir:
        suffix = os.path.basename(path.suffix)
    else:
        suffix = path.suffix
    return Path(suffix, install_root)



import json
import ntpath
import os
import uuid
from collections import defaultdict
from lxml import etree
from lxml.builder import E
from six import iteritems

from ... import path
from ... import shell
from ...safe_str import safe_str

__all__ = ['Solution', 'UuidMap', 'VcxProject']


def uuid_str(uuid):
    return '{{{}}}'.format(str(uuid).upper())


class SlnElement(object):
    def __init__(self, name, arg=None, value=None):
        if arg is not None and value is None:
            raise TypeError('if arg is passed, value must be too')
        self.name = name
        self.arg = arg
        self.value = value
        self.children = []

    def __call__(self, *args):
        return self.extend(args)

    def append(self, item):
        self.children.append(item)
        return self

    def extend(self, args):
        self.children.extend(args)
        return self

    def write(self, out, depth=0):
        out.write('\t' * depth)
        out.write(self.name)
        if self.arg:
            out.write('({}) = {}'.format(self.arg, self.value))
        out.write('\n')

        for i in self.children:
            i.write(out, depth + 1)

        out.write('\t' * depth + 'End' + self.name + '\n')


class SlnVariable(object):
    def __init__(self, name, value):
        self.name = name
        self.value = value

    def write(self, out, depth=0):
        out.write('\t' * depth + '{} = {}\n'.format(self.name, self.value))


class SlnBuilder(object):
    def __init__(self):
        pass

    def __call__(self, *args, **kwargs):
        return SlnElement(*args, **kwargs)

    def __getattribute__(self, name):
        def closure(*args, **kwargs):
            return SlnElement(name, *args, **kwargs)
        return closure


class Solution(object):
    def __init__(self, uuids):
        self.uuid = uuids['']
        self._uuids = uuids
        self._projects = []
        self._project_map = {}

    def __setitem__(self, key, value):
        value.set_uuid(self._uuids)
        self._projects.append(value)
        self._project_map[id(key)] = value

    def __getitem__(self, key):
        return self._project_map[id(key)]

    def __iter__(self):
        return iter(self._projects)

    def __contains__(self, key):
        return id(key) in self._project_map

    @property
    def uuid_str(self):
        return uuid_str(self.uuid)

    def write(self, out):
        S = SlnBuilder()
        Var = SlnVariable

        out.write('Microsoft Visual Studio Solution File, Format Version ' +
                  '12.00\n')
        out.write('# Visual Studio 14\n')

        Var('VisualStudioVersion', '14.0.22609.0').write(out)
        Var('MinimumVisualStudioVersion', '10.0.40219.1').write(out)

        configs = set()
        project_info = []

        for p in self._projects:
            proj = S.Project(
                '"{}"'.format(self.uuid_str),
                '"{name}", "{path}", "{uuid}"'.format(
                    name=p.name, path=p.path, uuid=p.uuid_str
                )
            )
            if p.dependencies:
                proj.append(
                    S.ProjectSection('ProjectDependencies', 'postProject')
                     .extend(Var(i.uuid_str, i.uuid_str)
                             for i in p.dependencies)
                )
            proj.write(out)

            configs.add(p.config_plat)
            project_info.append(Var('{uuid}.{cfg}.ActiveCfg'.format(
                uuid=p.uuid_str, cfg=p.config_plat
            ), p.config_plat))
            project_info.append(Var('{uuid}.{cfg}.Build.0'.format(
                uuid=p.uuid_str, cfg=p.config_plat
            ), p.config_plat))

        S.Global()(
            S.GlobalSection('SolutionConfigurationPlatforms', 'preSolution')
             .extend(Var(i, i) for i in configs),
            S.GlobalSection('ProjectConfigurationPlatforms', 'postSolution')
             .extend(project_info),
            S.GlobalSection('SolutionProperties', 'preSolution')(
                Var('HideSolutionNode', 'FALSE')
            )
        ).write(out)


_path_vars = {
    False: {
        path.Root.srcdir: '$(SourceDir)',
        path.Root.builddir: '$(IntDir)',
    },
    True: {
        path.Root.srcdir: '$(SourceDir)',
        path.Root.builddir: '$(OutDir)',
    },
}


def path_str(node, out=False):
    return ntpath.normpath(safe_str(node).realize(_path_vars[out]))


class VcxProject(object):
    _XMLNS = 'http://schemas.microsoft.com/developer/msbuild/2003'
    _DOCTYPE = '<?xml version="1.0" encoding="utf-8"?>'

    def __init__(self, name, uuid=None, version=None, mode='Application',
                 configuration=None, platform=None, output_file=None,
                 import_lib=None, srcdir=None, files=None,
                 compile_options=None, includes=None, link_options=None,
                 libs=None, lib_dirs=None, dependencies=None):
        self.name = name
        self.uuid = uuid
        self.version = version or '14.0'
        self.mode = mode
        self.configuration = configuration or 'Debug'
        self.platform = platform or 'Win32'
        self.output_file = output_file
        self.srcdir = srcdir
        self.files = files or []
        self.compile_options = compile_options or []
        self.includes = includes or []
        self.link_options = link_options or []
        self.libs = libs or []
        self.lib_dirs = lib_dirs or []
        self.dependencies = dependencies or []

    @property
    def path(self):
        return os.path.join(self.name, '{}.vcxproj'.format(
            os.path.basename(self.name)
        ))

    @property
    def config_plat(self):
        return '{config}|{platform}'.format(
            config=self.configuration,
            platform=self.platform
        )

    def set_uuid(self, uuids):
        if not self.uuid:
            self.uuid = uuids[self.name]

    @property
    def uuid_str(self):
        return uuid_str(self.uuid)

    @property
    def toolset(self):
        return 'v' + self.version.replace('.', '')

    def write(self, out):
        override_props = E.PropertyGroup()
        if self.lib_dirs:
            override_props.append(E.LibraryPath(
                ';'.join(self.lib_dirs + ['$(LibraryPath)'])
            ))
        override_props.append(E.TargetPath(
            path_str(self.output_file, out=True)
        ))

        compile_opts = E.ClCompile(
            E.WarningLevel('Level3')
        )
        if self.compile_options:
            compile_opts.append(E.AdditionalOptions(
                ' '.join(shell.quote(i) for i in self.compile_options) +
                ' %(AdditionalOptions)'
            ))
        if self.includes:
            compile_opts.append(E.AdditionalIncludeDirectories(
                ';'.join(path_str(i) for i in self.includes) +
                ';%(AdditionalIncludeDirectories)'
            ))

        link_opts = E.Lib() if self.mode == 'StaticLibrary' else E.Link()
        link_opts.append(E.OutputFile('$(TargetPath)'))
        if hasattr(self.output_file, 'import_lib'):
            link_opts.append(E.ImportLibrary(
                path_str(self.output_file.import_lib, out=True)
            ))
        if self.link_options:
            link_opts.append(E.AdditionalOptions(
                ' '.join(shell.quote(i) for i in self.link_options) +
                ' %(AdditionalOptions)'
            ))
        if self.libs:
            link_opts.append(E.AdditionalDependencies(
                ';'.join(path_str(i.link, out=True) for i in self.libs) +
                ';%(AdditionalDependencies)'
            ))

        names = defaultdict(lambda: [])
        for i in self.files:
            basename = ntpath.splitext(i.path.basename())[0]
            names[basename].append(i.path.parent())

        compiles = E.ItemGroup()
        for i in self.files:
            c = E.ClCompile(Include=path_str(i))
            dupes = names[ ntpath.splitext(i.path.basename())[0] ]
            if len(dupes) > 1:
                # XXX: This can still fail rarely if the paths' bases are
                # different.
                prefix = ntpath.commonprefix([j.suffix for j in dupes])
                suffix = path.Path(i.path.parent().suffix[len(prefix):])
                c.append(E.ObjectFileName(path_str(suffix) + '\\'))
            compiles.append(c)

        project = E.Project({'DefaultTargets': 'Build',
                             'ToolsVersion': self.version,
                             'xmlns': self._XMLNS},
            E.ItemGroup({'Label' : 'ProjectConfigurations'},
                E.ProjectConfiguration({'Include' : self.config_plat},
                    E.Configuration(self.configuration),
                    E.Platform(self.platform)
                )
            ),
            E.PropertyGroup({'Label': 'Globals'},
                E.ProjectGuid(self.uuid_str),
                E.RootNamespace(self.name),
                E.SourceDir(ntpath.normpath(self.srcdir))
            ),
            E.Import(Project='$(VCTargetsPath)\Microsoft.Cpp.default.props'),
            E.PropertyGroup({'Label': 'Configuration'},
                E.ConfigurationType(self.mode),
                E.UseDebugLibraries('true'),
                E.PlatformToolset(self.toolset),
                E.CharacterSet('Multibyte')
            ),
            E.Import(Project='$(VCTargetsPath)\Microsoft.Cpp.props'),
            override_props,
            E.ItemDefinitionGroup(compile_opts, link_opts),
            compiles,
            E.Import(Project='$(VCTargetsPath)\Microsoft.Cpp.Targets')
        )

        out.write(etree.tostring(project, doctype=self._DOCTYPE,
                                 pretty_print=True))


class UuidMap(object):
    version = 1

    def __init__(self, path):
        self._path = path
        self._seen = set()
        try:
            self._map = self._load(path)
        except IOError:
            self._map = {}

    def __getitem__(self, key):
        self._seen.add(key)
        if key in self._map:
            return self._map[key]
        else:
            u = uuid.uuid4()
            self._map[key] = u
            return u

    @classmethod
    def _load(cls, path):
        with open(path) as inp:
            state = json.load(inp)
        if state['version'] > cls.version:
            raise ValueError('saved version exceeds expected version')
        return { k: uuid.UUID(hex=v) for k, v in iteritems(state['map']) }

    def save(self, path=None):
        with open(path or self._path, 'w') as out:
            # Only save the UUIDs we saw this time. Skip ones we didn't see.
            seenmap = { k: v.hex for k, v in iteritems(self._map)
                        if k in self._seen }
            json.dump({
                'version': self.version,
                'map': seenmap,
            }, out)



import os

from .. import shell
from .. import file_types
from .. import iterutils
from ..path import Root


class ArLinker(object):
    rule_name = command_var = link_var = 'ar'

    def __init__(self, env, lang):
        self.platform = env.platform
        self.mode = 'static_library'
        self.lang = lang

        self.command = env.getvar('AR', 'ar')
        self.global_args = shell.split(env.getvar('ARFLAGS', 'cru'))

    @property
    def flavor(self):
        return 'ar'

    def __call__(self, cmd, input, output, args=None):
        result = [cmd]
        result.extend(iterutils.iterate(args))
        result.append(output)
        result.extend(iterutils.iterate(input))
        return result

    def output_file(self, name):
        head, tail = os.path.split(name)
        path = os.path.join(head, 'lib' + tail + '.a')
        return file_types.StaticLibrary(path, Root.builddir, self.lang)

    @property
    def mode_args(self):
        return []



import os.path
import re

from . import ar, cc, msvc
from .. import shell
from .hooks import builder
from ..languages import lang_exts


lang_exts('c', ['.c'])
lang_exts('c++', ['.cpp', '.cc', '.cp', '.cxx', '.CPP', '.c++', '.C'])

# XXX: Currently, we tie the linker to a single language, much like the
# compiler. However, linkers can generally take object files made from multiple
# source languages. We should figure out what the correct thing to do here is.


@builder('c', 'c++')
class CFamilyBuilder(object):
    __langs = {
        'c'  : ('CC' , 'cc' ),
        'c++': ('CXX', 'c++'),
    }

    def __init__(self, env, lang):
        self.linkers = {}

        var, default_cmd = self.__langs[lang]
        low_var = var.lower()
        if env.platform.name == 'windows':
            default_cmd = 'cl'
        cmd = env.getvar(var, default_cmd)

        cflags = (
            shell.split(env.getvar(var + 'FLAGS', '')) +
            shell.split(env.getvar('CPPFLAGS', ''))
        )
        ldflags = shell.split(env.getvar('LDFLAGS', ''))
        ldlibs = shell.split(env.getvar('LDLIBS', ''))

        if re.search(r'cl(\.exe)?$', cmd):
            origin = os.path.dirname(cmd)
            link_cmd = env.getvar(var + '_LINK', os.path.join(origin, 'link'))
            lib_cmd = env.getvar(var + '_LIB', os.path.join(origin, 'lib'))

            self.compiler = msvc.MsvcCompiler(env, lang, low_var, cmd, cflags)
            for mode in ['executable', 'shared_library']:
                self.linkers[mode] = msvc.MsvcLinker(
                    env, mode, lang, low_var, link_cmd, ldflags, ldlibs
                )
            self.linkers['static_library'] = msvc.MsvcStaticLinker(
                env, lang, low_var, lib_cmd
            )
            self.packages = msvc.MsvcPackageResolver(env, lang)
        else:
            self.compiler = cc.CcCompiler(env, lang, low_var, cmd, cflags)
            for mode in ['executable', 'shared_library']:
                self.linkers[mode] = cc.CcLinker(
                    env, mode, lang, low_var, cmd, ldflags, ldlibs
                )
            self.linkers['static_library'] = ar.ArLinker(env, lang)
            self.packages = cc.CcPackageResolver(env, lang, cmd)



import os.path

from .integration import *


class TestExtraDeps(IntegrationTest):
    def __init__(self, *args, **kwargs):
        IntegrationTest.__init__(self, 'extra_deps', *args, **kwargs)

    @skip_if_backend('msbuild')
    def test_build(self):
        self.build(executable('program'))
        self.assertOutput([executable('program')], 'hello, world!\n')

        self.assertExists(os.path.join(self.builddir, '1'))
        self.assertExists(os.path.join(self.builddir, '2'))
        self.assertExists(os.path.join(self.builddir, '3'))

    @skip_if_backend('msbuild')
    def test_touch_3(self):
        self.build('touch3')

        self.assertExists(os.path.join(self.builddir, '2'))
        self.assertExists(os.path.join(self.builddir, '3'))



import fnmatch
import os
import posixpath
import re

from .hooks import builtin
from .. import path
from ..iterutils import iterate
from ..backends.make import writer as make
from ..backends.ninja import writer as ninja
from ..backends.make.syntax import Writer, Syntax
from ..build_inputs import build_input


build_input('find_dirs')(set)
depfile_name = '.bfg_find_deps'


def write_depfile(path, output, seen_dirs, makeify=False):
    with open(path, 'w') as f:
        out = Writer(f)
        out.write(output, Syntax.target)
        out.write_literal(':')
        for i in seen_dirs:
            out.write_literal(' ')
            out.write(os.path.abspath(i), Syntax.dependency)
        out.write_literal('\n')
        if makeify:
            for i in seen_dirs:
                out.write(os.path.abspath(i), Syntax.target)
                out.write_literal(':\n')


def _listdir(path):
    dirs, nondirs = [], []
    try:
        names = os.listdir(path)
        for name in names:
            if os.path.isdir(os.path.join(path, name)):
                dirs.append(name)
            else:
                nondirs.append(name)
    except:
        pass
    return dirs, nondirs


def _walk_flat(top):
    yield (top,) + _listdir(top)


def _walk_recursive(top):
    dirs, nondirs = _listdir(top)
    yield top, dirs, nondirs
    for name in dirs:
        path = posixpath.join(top, name)
        if not os.path.islink(path):
            for i in _walk_recursive(path):
                yield i


def _find_files(paths, name, type, flat, filter):
    results = []
    seen_dirs = []

    # "Does the walker choose the path, or the path the walker?" - Garth Nix
    walker = _walk_flat if flat else _walk_recursive

    def _filter_in_place(files, type, func):
        for i in reversed(range( len(files) )):
            if not func(files[i], type):
                files.pop(i)

    def _filter_join(base, files, name):
        return (posixpath.join(base, i) for i in fnmatch.filter(files, name))

    for p in iterate(paths):
        if type != 'f' and fnmatch.fnmatch(p, name):
            results.append(p)

        generator = walker(p)
        for base, dirs, files in generator:
            if filter:
                _filter_in_place(dirs, 'd', filter)
                _filter_in_place(files, 'f', filter)

            seen_dirs.append(base)
            if type != 'f':
                results.extend(_filter_join(base, dirs, name))
            if type != 'd':
                results.extend(_filter_join(base, files, name))

    return results, seen_dirs


def find(path='.', name='*', type=None, flat=False):
    return _find_files(path, name, type, flat, None)[0]

known_platforms = ['posix', 'linux', 'darwin', 'cygwin', 'windows']


@builtin.globals('env')
def filter_by_platform(env, name, type):
    my_plat = set([env.platform.name, env.platform.flavor])
    ex = '|'.join(re.escape(i) for i in known_platforms if i not in my_plat)
    return re.search(r'(^|_)(' + ex + r')(\.[^\.])?$', name) is None


@builtin.globals('builtins', 'build_inputs', 'env')
def find_files(builtins, build_inputs, env, path='.', name='*', type=None,
               flat=False, filter=filter_by_platform, cache=True):
    if filter == filter_by_platform:
        filter = builtins['filter_by_platform']

    results, seen_dirs = _find_files(path, name, type, flat, filter)
    if cache:
        build_inputs['find_dirs'].update(seen_dirs)
    return results


@make.post_rule
def make_regenerate_rule(build_inputs, buildfile, env):
    bfg9000 = env.tool('bfg9000')
    bfgcmd = make.cmd_var(bfg9000, buildfile)
    bfgpath = path.Path('build.bfg', path.Root.srcdir)

    if build_inputs['find_dirs']:
        write_depfile(env.builddir.append(depfile_name).string(),
                      'Makefile', build_inputs['find_dirs'], makeify=True)
        buildfile.include(depfile_name)

    buildfile.rule(
        target=path.Path('Makefile'),
        deps=[bfgpath],
        recipe=[bfg9000.regenerate(bfgcmd, path.Path('.'))]
    )


@ninja.post_rule
def ninja_regenerate_rule(build_inputs, buildfile, env):
    bfg9000 = env.tool('bfg9000')
    bfgcmd = ninja.cmd_var(bfg9000, buildfile)
    bfgpath = path.Path('build.bfg', path.Root.srcdir)
    depfile = None

    if build_inputs['find_dirs']:
        write_depfile(env.builddir.append(depfile_name).string(),
                      'build.ninja', build_inputs['find_dirs'])
        depfile = depfile_name

    buildfile.rule(
        name='regenerate',
        command=bfg9000.regenerate(bfgcmd, path.Path('.')),
        generator=True,
        depfile=depfile,
    )
    buildfile.build(
        output=path.Path('build.ninja'),
        rule='regenerate',
        implicit=[bfgpath]
    )



import importlib
import pkgutil

from .hooks import bind  # noqa

# Import all the packages in this directory so their hooks get run.
for _, name, _ in pkgutil.walk_packages(__path__, '.'):
    importlib.import_module(name, __package__)



from collections import Iterable
from six import string_types


def isiterable(thing):
    return isinstance(thing, Iterable) and not isinstance(thing, string_types)


def iterate(thing):
    def generate_none():
        return
        yield

    def generate_one(x):
        yield x

    if thing is None:
        return generate_none()
    elif isiterable(thing):
        return iter(thing)
    else:
        return generate_one(thing)


def tween(iterable, delim, prefix=None, suffix=None):
    first = True
    for i in iterable:
        if first:
            first = False
            if prefix is not None:
                yield prefix
        else:
            yield delim
        yield i
    if not first and suffix is not None:
        yield suffix


def uniques(iterable):
    def generate_uniques(iterable):
        seen = set()
        for item in iterable:
            if item not in seen:
                seen.add(item)
                yield item
    return list(generate_uniques(iterable))


def listify(thing, always_copy=False):
    if not always_copy and type(thing) == list:
        return thing
    return list(iterate(thing))



_builders = {}
_tools = {}


def builder(*args):
    if len(args) == 0:
        raise TypeError('must provide at least one language')
    multi = len(args) > 1

    def wrapper(fn):
        for i in args:
            _builders[i] = (fn, multi)
        return fn
    return wrapper


def get_builder(lang, env):
    try:
        fn, multi = _builders[lang]
        return fn(env, lang) if multi else fn(env)
    except KeyError:
        raise ValueError('unknown language "{}"'.format(lang))


def tool(name):
    def wrapper(fn):
        _tools[name] = fn
        return fn
    return wrapper


def get_tool(name, env):
    try:
        return _tools[name](env)
    except KeyError:
        raise ValueError('unknown tool "{}"'.format(name))
